/******************************************************************************/
/*                                                                            */
/*  <<< Copyright >>>                                                         */
/*                                                                            */
/*  The "xmemo" software is copyright 1999 by Takao Abe.                      */
/*  All rights reserved.                                                      */
/*                                                                            */
/*  The "xmemo" software refers to the source code, the binary code,          */
/*  the documents, and all derivatives based on this software under           */
/*  the copyright law, that is a software containing a program or a           */
/*  portion of a program, regardless of modifications and/or translations.    */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  <<< Licence and redistribution >>>                                        */
/*                                                                            */
/*  The source code and the binary code are free software.                    */
/*  The use and redistribution of source code and binary code are permitted   */
/*  provided that the following conditions are met:                           */
/*  You may use the software for private use and commercial use,              */
/*  and may modify your copy or copies of the software or any portion of it.  */
/*                                                                            */
/*  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESSED OR IMPLIED            */
/*  WARRANTIES OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED        */
/*  WARRANTIES OF MERCHANTABLILITY AND FITNESS FOR A PARTICULAR PURPOSE.      */
/*  IN NO EVENT SHALL THE AUTHOR TAKAO ABE BE LIABLE FOR ANY DIRECT,          */
/*  INDIRECT, GENERAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES           */
/*  ( INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR       */
/*  SERVICES; LOSS OF USE, DATA OR PROFITS; OR BUSINESS INTERRUPTION )        */
/*  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,       */
/*  STRICT LIABILITY, OR TORT ( INCLUDING NEGLIGENCE OR OTHERWISE )           */
/*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED       */
/*  OF THE POSSIBILITY OF SUCH DAMAGE.                                        */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with or without     */
/*  modification, the whole program or a portion of it, are permitted         */
/*  provided that one retains the entire copyright notice properly, and       */
/*  displays or retains this license in the documentation and/or other        */
/*  materials provided with the distribution.                                 */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with modification,  */
/*  the whole program or a portion of it, are permitted provided that one     */
/*  declares that it is modified software and not original, and displays or   */
/*  describes any modifications in the documentation and/or other materials   */
/*  provided with the distribution.                                           */
/*                                                                            */
/*  This software and the name of the author must not be used to endorse or   */
/*  promote products derived from this software without prior written         */
/*  permission.                                                               */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  EMail   abetakao@bea.hi-ho.ne.jp                                          */
/*          HCA00763@nifty.ne.jp                                              */
/*                                                                            */
/*  URL     http://www.bea.hi-ho.ne.jp/abetakao/                              */
/*                                                                            */
/******************************************************************************/


#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<time.h>


#include	<X11/Intrinsic.h>
#include	<X11/StringDefs.h>
#include	<X11/Shell.h>

#include	<X11/Xaw/AsciiText.h>
#include	<X11/Xaw/Command.h>
#include	<X11/Xaw/Form.h>


#include	"xmemo.h"
#include	"xmemodata.h"
#include	"xmemofunc.h"


/******************************************************************************/
/*  Local variables                                                           */
/******************************************************************************/


static	Widget	wDlgMemoEdit ;
static	Widget	wDlgMemoEditForm ;
static	Widget	wDlgMemoEditMenu ;
static	Widget	wDlgMemoEditText ;
static	Widget	sDlgMemoEditCmd [ 4 ] ;


/******************************************************************************/
/*  EditSave_Click                                                            */
/******************************************************************************/

static void EditSave_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

int 	n ;
Arg 	xArg [ 2 ] ;
char	*pStr, *pBuf ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "EditSave_Click", NULL ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNstring, &pStr ) ; n ++ ;
	XtGetValues ( wDlgMemoEditText, xArg, n ) ;

	if ( pStr == NULL ) return ;

	pBuf = XtMalloc ( strlen ( pStr ) + 1 ) ;
	if ( pBuf == NULL ) {
		xmemo_ErrorLog ( "EditSave_Clock", "XtMalloc", NULL ) ;
		XtPopdown (  wDlgMemoEdit ) ;
		return ;
	}

	strcpy ( pBuf, pStr ) ;

	if ( pMemoActive->pText != NULL ) XtFree ( (void*) ( pMemoActive->pText ) ) ;
	pMemoActive->pText = pBuf ;

	XtPopdown ( wDlgMemoEdit ) ;

	if ( pMemoActive->pChain == NULL ) {

		xmemo_InsertMemo ( pMemoActive ) ;

	} else {

		n = 0 ;
		XtSetArg ( xArg[n], XtNstring, pBuf ) ; n ++ ;
		XtSetValues ( pMemoActive->wText, xArg, n ) ;

		xmemo_UpdateMemo ( pMemoActive ) ;
	}

}

static XtCallbackRec pEditSave_CallbackRec [ ] =
{
	{ (XtCallbackProc) EditSave_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  EditSend_Click                                                            */
/******************************************************************************/

static void EditSend_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

int 	n ;
Arg 	xArg [ 2 ] ;
char	*pStr, *pBuf ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "EditSend_Click", NULL ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNstring, &pStr ) ; n ++ ;
	XtGetValues ( wDlgMemoEditText, xArg, n ) ;

	if ( pStr == NULL ) return ;

	pBuf = XtMalloc ( strlen ( pStr ) + 1 ) ;
	if ( pBuf == NULL ) {
		xmemo_ErrorLog ( "EditSave_Clock", "XtMalloc", NULL ) ;
		XtPopdown (  wDlgMemoEdit ) ;
		return ;
	}

	strcpy ( pBuf, pStr ) ;

	if ( pMemoActive->pText != NULL ) XtFree ( (void*) ( pMemoActive->pText ) ) ;
	pMemoActive->pText = pBuf ;

	XtPopdown ( wDlgMemoEdit ) ;

	if ( pMemoActive->pChain != NULL ) {

		n = 0 ;
		XtSetArg ( xArg[n], XtNstring, pBuf ) ; n ++ ;
		XtSetValues ( pMemoActive->wText, xArg, n ) ;

		xmemo_UpdateMemo ( pMemoActive ) ;
	}

	xmemo_PopupDialog_Send ( ) ;

}

static XtCallbackRec pEditSend_CallbackRec [ ] =
{
	{ (XtCallbackProc) EditSend_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  EditAttribute_Click                                                          */
/******************************************************************************/

static void EditAttribute_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

	if ( bMemoTraceLog ) xmemo_TraceLog ( "EditAttribute_Click", NULL ) ;

	xmemo_PopupDialog_MemoAttribute ( pMemoActive ) ;

}

static XtCallbackRec pEditAttribute_CallbackRec [ ] =
{
	{ (XtCallbackProc) EditAttribute_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  EditCancel_Click                                                          */
/******************************************************************************/

static void EditCancel_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

	if ( bMemoTraceLog ) xmemo_TraceLog ( "EditCancel_Click", NULL ) ;

	XtPopdown ( wDlgMemoEdit ) ;

	if ( pMemoActive->pChain == NULL ) {

		xmemo_FreeMemo ( pMemoActive ) ;

	}

}

static XtCallbackRec pEditCancel_CallbackRec [ ] =
{
	{ (XtCallbackProc) EditCancel_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  xmemo_InitDialog_Edit                                                    */
/******************************************************************************/

int xmemo_InitDialog_Edit ( )
{

int 	n ;
Arg 	xArg [ 16 ] ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_InitDialog_Edit", NULL ) ;

	/* Dialog shell */

	n = 0 ;
	XtSetArg ( xArg[n], XtNtitle , "xmemo - Edit" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNwidth , 300 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNheight, 300 ) ; n ++ ;
	wDlgMemoEdit = XtCreatePopupShell ( "DlgEdit", transientShellWidgetClass, wTop, xArg, n ) ;
	                                  
	/* Main form */

	wDlgMemoEditForm = XtCreateManagedWidget ( "Form", formWidgetClass, wDlgMemoEdit, xArg, 0 ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNborderWidth, 0 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop   , XtChainTop ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom, XtChainTop ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft  , XtChainLeft ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright , XtChainLeft ) ; n ++ ;
	XtSetArg ( xArg[n], XtNheight, 30 ) ; n ++ ;
	wDlgMemoEditMenu = XtCreateManagedWidget ( "Menu", formWidgetClass, wDlgMemoEditForm, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNwidth , 300 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom, XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft  , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright , XtChainRight  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromVert, wDlgMemoEditMenu ) ; n ++ ;
	XtSetArg ( xArg[n], XtNeditType, XawtextEdit ) ; n ++ ;
	XtSetArg ( xArg[n], XtNscrollHorizontal, XawtextScrollWhenNeeded ) ; n ++ ;
	XtSetArg ( xArg[n], XtNscrollVertical  , XawtextScrollWhenNeeded ) ; n ++ ;
	/*XtSetArg ( xArg[n], XtNwrap            , XawtextWrapLine ) ; n ++ ;*/
	wDlgMemoEditText = XtCreateManagedWidget ( "Text", asciiTextWidgetClass, wDlgMemoEditForm, xArg, n ) ;

	/* Menu button */

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel    , "Save" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, NULL ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback , pEditSave_CallbackRec ) ; n ++ ;
	sDlgMemoEditCmd[0] = XtCreateManagedWidget ( "MenuButton", commandWidgetClass, wDlgMemoEditMenu, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel    , "Send" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, sDlgMemoEditCmd[0] ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback , pEditSend_CallbackRec ) ; n ++ ;
	sDlgMemoEditCmd[1] = XtCreateManagedWidget ( "MenuButton", commandWidgetClass, wDlgMemoEditMenu, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel    , "Attributes" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, sDlgMemoEditCmd[1] ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback , pEditAttribute_CallbackRec ) ; n ++ ;
	sDlgMemoEditCmd[2] = XtCreateManagedWidget ( "MenuButton", commandWidgetClass, wDlgMemoEditMenu, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel    , "Cancel" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, sDlgMemoEditCmd[2] ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback , pEditCancel_CallbackRec ) ; n ++ ;
	sDlgMemoEditCmd[3] = XtCreateManagedWidget ( "MenuButton", commandWidgetClass, wDlgMemoEditMenu, xArg, n ) ;

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_PopupDialog_Edit                                                    */
/******************************************************************************/

int xmemo_PopupDialog_Edit ( )
{

int 	n ;
Arg 	xArg [ 2 ] ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_PopupDialog_Edit", NULL ) ;

	n = 0 ;
	if ( pMemoActive->pText != NULL ) {
		XtSetArg ( xArg[n], XtNstring, pMemoActive->pText ) ; n ++ ;
	} else {
		XtSetArg ( xArg[n], XtNstring, "" ) ; n ++ ;
	}
	XtSetValues ( wDlgMemoEditText, xArg, n ) ;

	XtPopup ( wDlgMemoEdit, XtGrabExclusive ) ;

	return RC_SUCCESS ;
	
}

/******************************************************************************/
/*  xmemo_PopdownDialog_Edit                                                  */
/******************************************************************************/

int xmemo_PopdownDialog_Edit ( )
{

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_PopdownDialog_Edit", NULL ) ;

	XtPopdown ( wDlgMemoEdit ) ;

	return RC_SUCCESS ;

}
