/******************************************************************************/
/*                                                                            */
/*  <<< Copyright >>>                                                         */
/*                                                                            */
/*  The "xmemo" software is copyright 1999 by Takao Abe.                      */
/*  All rights reserved.                                                      */
/*                                                                            */
/*  The "xmemo" software refers to the source code, the binary code,          */
/*  the documents, and all derivatives based on this software under           */
/*  the copyright law, that is a software containing a program or a           */
/*  portion of a program, regardless of modifications and/or translations.    */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  <<< Licence and redistribution >>>                                        */
/*                                                                            */
/*  The source code and the binary code are free software.                    */
/*  The use and redistribution of source code and binary code are permitted   */
/*  provided that the following conditions are met:                           */
/*  You may use the software for private use and commercial use,              */
/*  and may modify your copy or copies of the software or any portion of it.  */
/*                                                                            */
/*  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESSED OR IMPLIED            */
/*  WARRANTIES OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED        */
/*  WARRANTIES OF MERCHANTABLILITY AND FITNESS FOR A PARTICULAR PURPOSE.      */
/*  IN NO EVENT SHALL THE AUTHOR TAKAO ABE BE LIABLE FOR ANY DIRECT,          */
/*  INDIRECT, GENERAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES           */
/*  ( INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR       */
/*  SERVICES; LOSS OF USE, DATA OR PROFITS; OR BUSINESS INTERRUPTION )        */
/*  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,       */
/*  STRICT LIABILITY, OR TORT ( INCLUDING NEGLIGENCE OR OTHERWISE )           */
/*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED       */
/*  OF THE POSSIBILITY OF SUCH DAMAGE.                                        */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with or without     */
/*  modification, the whole program or a portion of it, are permitted         */
/*  provided that one retains the entire copyright notice properly, and       */
/*  displays or retains this license in the documentation and/or other        */
/*  materials provided with the distribution.                                 */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with modification,  */
/*  the whole program or a portion of it, are permitted provided that one     */
/*  declares that it is modified software and not original, and displays or   */
/*  describes any modifications in the documentation and/or other materials   */
/*  provided with the distribution.                                           */
/*                                                                            */
/*  This software and the name of the author must not be used to endorse or   */
/*  promote products derived from this software without prior written         */
/*  permission.                                                               */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  EMail   abetakao@bea.hi-ho.ne.jp                                          */
/*          HCA00763@nifty.ne.jp                                              */
/*                                                                            */
/*  URL     http://www.bea.hi-ho.ne.jp/abetakao/                              */
/*                                                                            */
/******************************************************************************/


#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<time.h>


#include	<X11/Intrinsic.h>
#include	<X11/StringDefs.h>
#include	<X11/Shell.h>

#include	<X11/Xaw/AsciiText.h>
#include	<X11/Xaw/Command.h>
#include	<X11/Xaw/Form.h>
#include	<X11/Xaw/List.h>
#include	<X11/Xaw/Viewport.h>


#include	"xmemo.h"
#include	"xmemodata.h"
#include	"xmemofunc.h"


/******************************************************************************/
/*  Local variables                                                           */
/******************************************************************************/


static	Widget	wDlgFont ;
static	Widget	wDlgFontForm ;
static	Widget	wDlgFontLblSearch ;
static	Widget	wDlgFontTxtSearch ;
static	Widget	wDlgFontCmdSearch ;
static	Widget	wDlgFontViewport ;
static	Widget	wDlgFontList ;
static	Widget	wDlgFontCmdClose ;
static	Widget	wDlgFontCmdCancel ;


static	char	**pFontList ;
static	int 	iFontCount ;


/******************************************************************************/
/*  FontSearch_Click                                                          */
/******************************************************************************/

static void FontSearch_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

int 	n ;
Arg 	xArg [ 5 ] ;
char	*pSearch ;
XawListReturnStruct 	*pListSelect ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "FontSearch_Click", NULL ) ;

	if ( pFontList != NULL ) XFreeFontNames ( pFontList ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNstring, &pSearch ) ; n ++ ;
	XtGetValues ( wDlgFontTxtSearch, xArg, n ) ;

	pFontList = XListFonts ( XtDisplay(wTop), pSearch, 1000, &iFontCount ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNlist, pFontList ) ; n ++ ;
	XtSetArg ( xArg[n], XtNnumberStrings, iFontCount ) ; n ++ ;
	XtSetValues ( wDlgFontList, xArg, n ) ;
	XawListChange ( wDlgFontList, pFontList, iFontCount, 0, False ) ;

}

static XtCallbackRec pFontSearch_CallbackRec [ ] =
{
	{ (XtCallbackProc) FontSearch_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  FontClose_Click                                                           */
/******************************************************************************/

static void FontClose_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

XawListReturnStruct 	*pListSelect ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "FontClose_Click", NULL ) ;

	/* Input */

	pListSelect = XawListShowCurrent ( wDlgFontList ) ;

	/* Check */

	if ( pListSelect == NULL ) {
		XBell ( XtDisplay(wTop), 100 ) ;
		return ;
	}

	if ( pListSelect->list_index < 0 ) {
		XBell ( XtDisplay(wTop), 100 ) ;
		return ;
	}

	/* Save */

	xmemo_DlgOptionPreference_SetFont ( pListSelect->string ) ;

	/* Close dialog */

	xmemo_PopdownDialog_Font ( ) ;

}

static XtCallbackRec pFontClose_CallbackRec [ ] =
{
	{ (XtCallbackProc) FontClose_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  FontCancel_Click                                                          */
/******************************************************************************/

static void FontCancel_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

	if ( bMemoTraceLog ) xmemo_TraceLog ( "FontCancel_Click", NULL ) ;

	xmemo_PopdownDialog_Font ( ) ;

}

static XtCallbackRec pFontCancel_CallbackRec [ ] =
{
	{ (XtCallbackProc) FontCancel_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  xmemo_InitDialog_Font                                                     */
/******************************************************************************/

int xmemo_InitDialog_Font ( )
{

int 	n ;
Arg 	xArg [ 16 ] ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_InitDialog_Font", NULL ) ;

	pFontList = NULL ;

	/* Dialog shell */

	n = 0 ;
	XtSetArg ( xArg[n], XtNtitle , "xmemo - Font" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNwidth , 400 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNheight, 300 ) ; n ++ ;
	wDlgFont = XtCreatePopupShell ( "DlgFont", transientShellWidgetClass, wTop, xArg, n ) ;
	                                  
	/* Main form */

	wDlgFontForm = XtCreateManagedWidget ( "Form", formWidgetClass, wDlgFont, xArg, 0 ) ;

	/* List area */

	n = 0 ;
	XtSetArg ( xArg[n], XtNwidth   , 100 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNlabel   , "Search name" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNjustify , XtJustifyLeft ) ; n ++ ;
	XtSetArg ( xArg[n], XtNborderWidth, 0 ) ; n ++ ;	
	XtSetArg ( xArg[n], XtNtop   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom, XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft  , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright , XtChainLeft   ) ; n ++ ;
	wDlgFontLblSearch = XtCreateManagedWidget ( "Label", labelWidgetClass, wDlgFontForm, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNwidth    , 200 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNstring   , "*" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop      , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft     , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright    , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, wDlgFontLblSearch ) ; n ++ ;	
	XtSetArg ( xArg[n], XtNeditType , XawtextEdit   ) ; n ++ ;	
	wDlgFontTxtSearch = XtCreateManagedWidget ( "Text", asciiTextWidgetClass, wDlgFontForm, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel    , "Search" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop      , XtChainTop  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom   , XtChainTop  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft     , XtChainLeft ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright    , XtChainLeft ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, wDlgFontTxtSearch ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback , pFontSearch_CallbackRec ) ; n ++ ;
	wDlgFontCmdSearch = XtCreateManagedWidget ( "CmdButton", commandWidgetClass, wDlgFontForm, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNtop   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom, XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft  , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright , XtChainRight  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromVert  , wDlgFontLblSearch ) ; n ++ ;
	XtSetArg ( xArg[n], XtNallowHoriz, False ) ; n ++ ;
	XtSetArg ( xArg[n], XtNallowVert , True  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNuseBottom , True  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNuseRight  , True  ) ; n ++ ;
	wDlgFontViewport = XtCreateManagedWidget ( "ListViewport", viewportWidgetClass, wDlgFontForm, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNtop   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom, XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft  , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright , XtChainRight  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNdefaultColumns, 1     ) ; n ++ ;
	XtSetArg ( xArg[n], XtNforceColumns  , TRUE  ) ; n ++ ;
	wDlgFontList = XtCreateManagedWidget ( "List", listWidgetClass, wDlgFontViewport, xArg, n ) ;

	/* Command button ( Bottom ) */

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel    , "OK" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop      , XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom   , XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft     , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright    , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromVert , wDlgFontViewport ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback , pFontClose_CallbackRec ) ; n ++ ;
	wDlgFontCmdClose = XtCreateManagedWidget ( "CmdButton", commandWidgetClass, wDlgFontForm, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel    , "Cancel" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop      , XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom   , XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft     , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright    , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromVert , wDlgFontViewport ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, wDlgFontCmdClose ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback , pFontCancel_CallbackRec ) ; n ++ ;
	wDlgFontCmdCancel = XtCreateManagedWidget ( "CmdButton", commandWidgetClass, wDlgFontForm, xArg, n ) ;

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_PopupDialog_Font                                                    */
/******************************************************************************/

int xmemo_PopupDialog_Font ( )
{

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_PopupDialog_Font", NULL ) ;

	FontSearch_Click ( wDlgFontCmdSearch, NULL, NULL ) ;

	XtPopup ( wDlgFont, XtGrabExclusive ) ;

	return RC_SUCCESS ;
	
}

/******************************************************************************/
/*  xmemo_PopdownDialog_Font                                                  */
/******************************************************************************/

int xmemo_PopdownDialog_Font ( )
{

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_PopdownDialog_Font", NULL ) ;

	XtPopdown ( wDlgFont ) ;

	XFreeFontNames ( pFontList ) ;

	pFontList = NULL ;

	return RC_SUCCESS ;

}
