/******************************************************************************/
/*                                                                            */
/*  <<< Copyright >>>                                                         */
/*                                                                            */
/*  The "xmemo" software is copyright 1999 by Takao Abe.                      */
/*  All rights reserved.                                                      */
/*                                                                            */
/*  The "xmemo" software refers to the source code, the binary code,          */
/*  the documents, and all derivatives based on this software under           */
/*  the copyright law, that is a software containing a program or a           */
/*  portion of a program, regardless of modifications and/or translations.    */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  <<< Licence and redistribution >>>                                        */
/*                                                                            */
/*  The source code and the binary code are free software.                    */
/*  The use and redistribution of source code and binary code are permitted   */
/*  provided that the following conditions are met:                           */
/*  You may use the software for private use and commercial use,              */
/*  and may modify your copy or copies of the software or any portion of it.  */
/*                                                                            */
/*  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESSED OR IMPLIED            */
/*  WARRANTIES OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED        */
/*  WARRANTIES OF MERCHANTABLILITY AND FITNESS FOR A PARTICULAR PURPOSE.      */
/*  IN NO EVENT SHALL THE AUTHOR TAKAO ABE BE LIABLE FOR ANY DIRECT,          */
/*  INDIRECT, GENERAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES           */
/*  ( INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR       */
/*  SERVICES; LOSS OF USE, DATA OR PROFITS; OR BUSINESS INTERRUPTION )        */
/*  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,       */
/*  STRICT LIABILITY, OR TORT ( INCLUDING NEGLIGENCE OR OTHERWISE )           */
/*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED       */
/*  OF THE POSSIBILITY OF SUCH DAMAGE.                                        */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with or without     */
/*  modification, the whole program or a portion of it, are permitted         */
/*  provided that one retains the entire copyright notice properly, and       */
/*  displays or retains this license in the documentation and/or other        */
/*  materials provided with the distribution.                                 */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with modification,  */
/*  the whole program or a portion of it, are permitted provided that one     */
/*  declares that it is modified software and not original, and displays or   */
/*  describes any modifications in the documentation and/or other materials   */
/*  provided with the distribution.                                           */
/*                                                                            */
/*  This software and the name of the author must not be used to endorse or   */
/*  promote products derived from this software without prior written         */
/*  permission.                                                               */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  EMail   abetakao@bea.hi-ho.ne.jp                                          */
/*          HCA00763@nifty.ne.jp                                              */
/*                                                                            */
/*  URL     http://www.bea.hi-ho.ne.jp/abetakao/                              */
/*                                                                            */
/******************************************************************************/


#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<time.h>


#include	<X11/Intrinsic.h>
#include	<X11/StringDefs.h>
#include	<X11/Shell.h>

#include	<X11/Xaw/AsciiText.h>
#include	<X11/Xaw/Command.h>
#include	<X11/Xaw/Form.h>
#include	<X11/Xaw/Label.h>
#include	<X11/Xaw/Toggle.h>


#include	"xmemo.h"
#include	"xmemodata.h"
#include	"xmemofunc.h"


/******************************************************************************/
/*  Local variables                                                           */
/******************************************************************************/


static	Widget	wDlgOptionSystem ;
static	Widget	wDlgOptionSystemForm ;
static	Widget	wDlgOptionSystemEdit ;
static	Widget	wDlgOptionSystemLbl [ 2 ] ;
static	Widget	wDlgOptionSystemTglCharset [ 5 ] ;
static	Widget	wDlgOptionSystemCmdClose ;
static	Widget	wDlgOptionSystemCmdCancel ;


/******************************************************************************/
/*  OptionSystemClose_Click                                                   */
/******************************************************************************/

static void OptionSystemClose_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

int 	i ;
int 	n ;
Arg 	xArg [ 5 ] ;
int 	bState ;
int 	bTglCharset [ 5 ] ;
char	cTglCharset ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "OptionSystemClose_Click", NULL ) ;

	/* Input */

	for ( i = 0 ; i < 5 ; i ++ ) {
		n = 0 ;
		XtSetArg ( xArg[n], XtNstate, &bState ) ; n ++ ;
		XtGetValues ( wDlgOptionSystemTglCharset[i], xArg, n ) ;
		bTglCharset[i] = ( ( bState & True ) == True ) ;
	}
	if      ( bTglCharset[0] ) cTglCharset = CHARSET_ASCII ;
	else if ( bTglCharset[1] ) cTglCharset = CHARSET_EUC ;
	else if ( bTglCharset[2] ) cTglCharset = CHARSET_JIS7 ;
	else if ( bTglCharset[3] ) cTglCharset = CHARSET_JIS8 ;
	else if ( bTglCharset[4] ) cTglCharset = CHARSET_SJIS ;
	else                       cTglCharset = CHARSET_EUC ;

	/* Check */

	/* Close dialog */

	xmemo_PopdownDialog_OptionSystem ( ) ;

	/* Save */

	cMemoCharset = cTglCharset ;

	xmemo_SaveOptionSystem ( ) ;

}

static XtCallbackRec pOptionSystemClose_CallbackRec [ ] =
{
	{ (XtCallbackProc) OptionSystemClose_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  OptionSystemCancel_Click                                                  */
/******************************************************************************/

static void OptionSystemCancel_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

zChainType	*pChain ;
zNetworkType	*pNetwork ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "OptionSystemCancel_Click", NULL ) ;

	xmemo_PopdownDialog_OptionSystem ( ) ;

}

static XtCallbackRec pOptionSystemCancel_CallbackRec [ ] =
{
	{ (XtCallbackProc) OptionSystemCancel_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  xmemo_InitDialog_OptionSystem                                             */
/******************************************************************************/

int xmemo_InitDialog_OptionSystem ( )
{

int 	n ;
Arg 	xArg [ 16 ] ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_InitDialog_OptionSystem", NULL ) ;

	/* Dialog shell */

	n = 0 ;
	XtSetArg ( xArg[n], XtNtitle , "xmemo - Option [ System ]" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNwidth , 400 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNheight, 200 ) ; n ++ ;
	wDlgOptionSystem = XtCreatePopupShell ( "DlgOptSystem", transientShellWidgetClass, wTop, xArg, n ) ;
	                                  
	/* Main form */

	wDlgOptionSystemForm = XtCreateManagedWidget ( "Form", formWidgetClass, wDlgOptionSystem, xArg, 0 ) ;

	/* Edit area */

	n = 0 ;
	XtSetArg ( xArg[n], XtNtop   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom, XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft  , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright , XtChainRight  ) ; n ++ ;
	wDlgOptionSystemEdit = XtCreateManagedWidget ( "Body", formWidgetClass, wDlgOptionSystemForm, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNwidth   , 100 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNlabel   , "Charset" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNjustify , XtJustifyLeft ) ; n ++ ;
	XtSetArg ( xArg[n], XtNborderWidth, 0 ) ; n ++ ;	
	XtSetArg ( xArg[n], XtNtop   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom, XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft  , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright , XtChainLeft   ) ; n ++ ;
	wDlgOptionSystemLbl[0] = XtCreateManagedWidget ( "Label", labelWidgetClass, wDlgOptionSystemEdit, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNwidth    , 50 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNlabel    , "ASCII" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop      , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft     , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright    , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromVert , NULL ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, wDlgOptionSystemLbl[0] ) ; n ++ ;	
	wDlgOptionSystemTglCharset[0] = XtCreateManagedWidget ( "TglButton", toggleWidgetClass, wDlgOptionSystemEdit, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNwidth    , 50 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNlabel    , "EUC" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop      , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft     , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright    , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromVert , NULL ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, wDlgOptionSystemTglCharset[0] ) ; n ++ ;	
	wDlgOptionSystemTglCharset[1] = XtCreateManagedWidget ( "TglButton", toggleWidgetClass, wDlgOptionSystemEdit, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNwidth    , 50 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNlabel    , "JIS7" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop      , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft     , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright    , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromVert , NULL ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, wDlgOptionSystemTglCharset[1] ) ; n ++ ;	
	wDlgOptionSystemTglCharset[2] = XtCreateManagedWidget ( "TglButton", toggleWidgetClass, wDlgOptionSystemEdit, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNwidth    , 50 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNlabel    , "JIS8" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop      , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft     , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright    , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromVert , NULL ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, wDlgOptionSystemTglCharset[2] ) ; n ++ ;	
	wDlgOptionSystemTglCharset[3] = XtCreateManagedWidget ( "TglButton", toggleWidgetClass, wDlgOptionSystemEdit, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNwidth    , 50 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNlabel    , "SJIS" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop      , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft     , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright    , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromVert , NULL ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, wDlgOptionSystemTglCharset[3] ) ; n ++ ;	
	wDlgOptionSystemTglCharset[4] = XtCreateManagedWidget ( "TglButton", toggleWidgetClass, wDlgOptionSystemEdit, xArg, n ) ;

	XawToggleChangeRadioGroup ( wDlgOptionSystemTglCharset[0], wDlgOptionSystemTglCharset[4] ) ;
	XawToggleChangeRadioGroup ( wDlgOptionSystemTglCharset[1], wDlgOptionSystemTglCharset[0] ) ;
	XawToggleChangeRadioGroup ( wDlgOptionSystemTglCharset[2], wDlgOptionSystemTglCharset[1] ) ;
	XawToggleChangeRadioGroup ( wDlgOptionSystemTglCharset[3], wDlgOptionSystemTglCharset[2] ) ;
	XawToggleChangeRadioGroup ( wDlgOptionSystemTglCharset[4], wDlgOptionSystemTglCharset[3] ) ;

	/* Command button ( Bottom ) */

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel    , "OK" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop      , XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom   , XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft     , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright    , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromVert , wDlgOptionSystemEdit ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback , pOptionSystemClose_CallbackRec ) ; n ++ ;
	wDlgOptionSystemCmdClose = XtCreateManagedWidget ( "CmdButton", commandWidgetClass, wDlgOptionSystemForm, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel    , "Cancel" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop      , XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom   , XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft     , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright    , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromVert , wDlgOptionSystemEdit ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, wDlgOptionSystemCmdClose ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback , pOptionSystemCancel_CallbackRec ) ; n ++ ;
	wDlgOptionSystemCmdCancel = XtCreateManagedWidget ( "CmdButton", commandWidgetClass, wDlgOptionSystemForm, xArg, n ) ;

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_PopupDialog_OptionSystem                                            */
/******************************************************************************/

int xmemo_PopupDialog_OptionSystem ( )
{

int 	n ;
Arg 	xArg [ 5 ] ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_PopupDialog_OptionSystem", NULL ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNstate, True  ) ; n ++ ;
	switch ( cMemoCharset ) {
	case CHARSET_ASCII : XtSetValues ( wDlgOptionSystemTglCharset[0], xArg, n ) ; break ;
	case CHARSET_EUC   : XtSetValues ( wDlgOptionSystemTglCharset[1], xArg, n ) ; break ;
	case CHARSET_JIS7  : XtSetValues ( wDlgOptionSystemTglCharset[2], xArg, n ) ; break ;
	case CHARSET_JIS8  : XtSetValues ( wDlgOptionSystemTglCharset[3], xArg, n ) ; break ;
	case CHARSET_SJIS  : XtSetValues ( wDlgOptionSystemTglCharset[4], xArg, n ) ; break ;
	}

	XtPopup ( wDlgOptionSystem, XtGrabExclusive ) ;

	return RC_SUCCESS ;
	
}

/******************************************************************************/
/*  xmemo_PopdownDialog_OptionSystem                                          */
/******************************************************************************/

int xmemo_PopdownDialog_OptionSystem ( )
{

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_PopdownDialog_OptionSystem", NULL ) ;

	XtPopdown ( wDlgOptionSystem ) ;

	return RC_SUCCESS ;

}
