/******************************************************************************/
/*                                                                            */
/*  <<< Copyright >>>                                                         */
/*                                                                            */
/*  The "xmemo" software is copyright 1999 by Takao Abe.                      */
/*  All rights reserved.                                                      */
/*                                                                            */
/*  The "xmemo" software refers to the source code, the binary code,          */
/*  the documents, and all derivatives based on this software under           */
/*  the copyright law, that is a software containing a program or a           */
/*  portion of a program, regardless of modifications and/or translations.    */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  <<< Licence and redistribution >>>                                        */
/*                                                                            */
/*  The source code and the binary code are free software.                    */
/*  The use and redistribution of source code and binary code are permitted   */
/*  provided that the following conditions are met:                           */
/*  You may use the software for private use and commercial use,              */
/*  and may modify your copy or copies of the software or any portion of it.  */
/*                                                                            */
/*  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESSED OR IMPLIED            */
/*  WARRANTIES OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED        */
/*  WARRANTIES OF MERCHANTABLILITY AND FITNESS FOR A PARTICULAR PURPOSE.      */
/*  IN NO EVENT SHALL THE AUTHOR TAKAO ABE BE LIABLE FOR ANY DIRECT,          */
/*  INDIRECT, GENERAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES           */
/*  ( INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR       */
/*  SERVICES; LOSS OF USE, DATA OR PROFITS; OR BUSINESS INTERRUPTION )        */
/*  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,       */
/*  STRICT LIABILITY, OR TORT ( INCLUDING NEGLIGENCE OR OTHERWISE )           */
/*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED       */
/*  OF THE POSSIBILITY OF SUCH DAMAGE.                                        */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with or without     */
/*  modification, the whole program or a portion of it, are permitted         */
/*  provided that one retains the entire copyright notice properly, and       */
/*  displays or retains this license in the documentation and/or other        */
/*  materials provided with the distribution.                                 */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with modification,  */
/*  the whole program or a portion of it, are permitted provided that one     */
/*  declares that it is modified software and not original, and displays or   */
/*  describes any modifications in the documentation and/or other materials   */
/*  provided with the distribution.                                           */
/*                                                                            */
/*  This software and the name of the author must not be used to endorse or   */
/*  promote products derived from this software without prior written         */
/*  permission.                                                               */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  EMail   abetakao@bea.hi-ho.ne.jp                                          */
/*          HCA00763@nifty.ne.jp                                              */
/*                                                                            */
/*  URL     http://www.bea.hi-ho.ne.jp/abetakao/                              */
/*                                                                            */
/******************************************************************************/


#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<time.h>


#include	<X11/Intrinsic.h>
#include	<X11/StringDefs.h>
#include	<X11/Shell.h>

#include	<X11/Xaw/Box.h>
#include	<X11/Xaw/Command.h>
#include	<X11/Xaw/Form.h>
#include	<X11/Xaw/List.h>
#include	<X11/Xaw/Viewport.h>


#include	"xmemo.h"
#include	"xmemodata.h"
#include	"xmemofunc.h"


/******************************************************************************/
/*  Local variables                                                           */
/******************************************************************************/


static	Widget	wDlgMemoSend ;
static	Widget	wDlgMemoSendForm ;
static	Widget	wDlgMemoSendMenu ;
static	Widget	wDlgMemoSendViewportGroup ;
static	Widget	wDlgMemoSendListGroup ;
static	Widget	wDlgMemoSendViewportUser ;
static	Widget	wDlgMemoSendListUser ;


static	int 	iGroupCount ;
static	char	**pGroupArray ;


static	int 	iListCount ;
static	int 	iListSelect ;
static	char	**pListArray ;
static	zMemberType	**pMemberArray ;


static	char	bRefreshTimer = FALSE ;
static	int 	iRefreshTimer ;


/******************************************************************************/
/*  xmemo_FreeMember                                                          */
/******************************************************************************/

void xmemo_FreeMember ( zMemberType *pMember )
{

	if ( pMember->pHost  != NULL ) XtFree ( (void*) ( pMember->pHost  ) ) ;
	if ( pMember->pUser  != NULL ) XtFree ( (void*) ( pMember->pUser  ) ) ;
	if ( pMember->pNick  != NULL ) XtFree ( (void*) ( pMember->pNick  ) ) ;
	if ( pMember->pGroup != NULL ) XtFree ( (void*) ( pMember->pGroup ) ) ;
	if ( pMember->pLine  != NULL ) XtFree ( (void*) ( pMember->pLine  ) ) ;
	XtFree ( (void*) pMember ) ;

}

/******************************************************************************/
/*  xmemo_SetGroupListArray                                                   */
/******************************************************************************/

static void xmemo_SetGroupListArray ( )
{

zChainType	*pChain ;
zMemberType	*pMember ;
int 	n ;
Arg 	xArg [ 3 ] ;
int 	i, j ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_SetGroupListArray", NULL ) ;

	/* Free old list */

	if ( pGroupArray != NULL ) {
		XtFree ( (void*) pGroupArray ) ;
	}

	/* Count effective entry */

	iGroupCount = 1 ;
	for ( pChain = zMemberAnchor.pHead ; pChain != NULL ; pChain = pChain->pNext ) {

		pMember = (zMemberType*) ( pChain->pData ) ;

		if ( ! pMember->bDelete ) iGroupCount ++ ;

	}

	/* Allocate memory */

	pGroupArray = (char**) XtCalloc ( sizeof(char*), iGroupCount + 1 ) ;
	if ( pGroupArray == NULL ) {
		xmemo_ErrorLog ( "xmemo_SetMemberListArray", "XtCalloc", NULL ) ;
		return ;
	}

	/* Set list entry ( Sorted by group name ) */

	pGroupArray[0] = "*ALL*" ;
	iGroupCount = 1 ;
	for ( pChain = zMemberAnchor.pHead ; pChain != NULL ; pChain = pChain->pNext ) {

		pMember = (zMemberType*) ( pChain->pData ) ;

		if ( ! pMember->bDelete  &&  pMember->pGroup != NULL ) {

			for ( i = 1 ; i < iGroupCount ; i ++ ) {
				if ( strcmp ( pGroupArray[i], pMember->pGroup ) >= 0 ) break ;
			}
			if ( i < iGroupCount ) {
				if ( strcmp ( pGroupArray[i], pMember->pGroup ) == 0 ) continue ;
			} 
			for ( j = iGroupCount - 1 ; j >= i ; j -- ) {
				pGroupArray[j+1] = pGroupArray[j] ;
			}
			pGroupArray[i] = pMember->pGroup ;
			iGroupCount ++ ;

		}

	}
	pGroupArray[iGroupCount] = NULL ;

	/* Refresh list widget */

	n = 0 ;
	XtSetArg ( xArg[n], XtNlist, pGroupArray ) ; n ++ ;
	XtSetArg ( xArg[n], XtNnumberStrings, iGroupCount ) ; n ++ ;
	XtSetValues ( wDlgMemoSendListGroup, xArg, n ) ;
	XawListChange ( wDlgMemoSendListGroup, pGroupArray, iGroupCount, 0, False ) ;

}

/******************************************************************************/
/*  xmemo_SetMemberListLine                                                   */
/******************************************************************************/

static void xmemo_SetMemberListLine ( zMemberType *pMember )
{

char	sBuf [ 84 ] ;
char	*pUsername, *pGroupname, *pHostname ;

	pMember->pLine = XtMalloc ( 84 ) ;
	if ( pMember->pLine == NULL ) {
		xmemo_ErrorLog ( "xmemo_SetMemberListLine", "XtMalloc", NULL ) ;
		return ;
	}

	if       ( pMember->pNick != NULL ) pUsername = pMember->pNick ;
	else if  ( pMember->pUser != NULL ) pUsername = pMember->pUser ;
	else if  ( pMember->pHost != NULL ) pUsername = pMember->pHost ;
	else                                pUsername = "" ;

	if       ( pMember->pGroup != NULL ) pGroupname = pMember->pGroup ;
	else                                 pGroupname = "" ;

	if       ( pMember->pHost != NULL ) pHostname = pMember->pHost ;
	else                                pHostname = "" ;

	/* Length = 2 ( Select mark ) */
	if ( pMember->bSelect ) strcpy ( pMember->pLine, "* " ) ;
	else                    strcpy ( pMember->pLine, "  " ) ;

	/* Length = 1 ( Absence ) */
	if ( pMember->bAbsence ) strcat ( pMember->pLine, "?" ) ;
	else                     strcat ( pMember->pLine, " " ) ;


	/* Length = 16 ( User ) */
	if ( strlen ( pUsername ) > 16 ) strncpy ( sBuf, pUsername, 16 ) ;
	else                             sprintf ( sBuf, "%-16s", pUsername ) ;
	sBuf[16] = 0 ;
	strcat ( pMember->pLine, sBuf ) ;

	/* Length = 2 ( Spaces ) */
	strcat ( pMember->pLine, "  " ) ;

	/* Length = 16 to 32 ( Group ) */
	if ( strlen ( pGroupname ) > 32 ) strncpy ( sBuf, pGroupname, 32 ) ;
	else                              sprintf ( sBuf, "%-16s", pGroupname ) ;
	sBuf[32] = 0 ;
	strcat ( pMember->pLine, sBuf ) ;

	/* Length = 2 ( Spaces ) */
	strcat ( pMember->pLine, "  " ) ;

	/* Length = 9 ( Protocol ) */
	switch ( pMember->cProtocol ) {
	case XMEMO_PROTOCOL_IPMSG    : strcat ( pMember->pLine, "IPMsg    " ) ; break ;
	case XMEMO_PROTOCOL_OBOEGAKI : strcat ( pMember->pLine, "Oboegaki " ) ; break ;
	case XMEMO_PROTOCOL_HUSEN    : strcat ( pMember->pLine, "Husen9X  " ) ; break ;
	default                      : strcat ( pMember->pLine, "Invalid  " ) ; break ;
	}

	/* Length = 2 ( Spaces ) */
	strcat ( pMember->pLine, "  " ) ;

	/* Length = 16 ( Host ) */
	if ( strlen ( pHostname ) > 16 ) strncpy ( sBuf, pHostname, 16 ) ;
	else                             sprintf ( sBuf, "%-16s", pHostname ) ;
	sBuf[16] = 0 ;
	strcat ( pMember->pLine, sBuf ) ;

}

/******************************************************************************/
/*  xmemo_SetMemberListArray                                                  */
/******************************************************************************/

static void xmemo_SetMemberListArray ( )
{

zChainType	*pChain ;
zMemberType	*pMember ;
int 	n ;
Arg 	xArg [ 3 ] ;
int 	i, j ;
int 	iGroupSelect ;
XawListReturnStruct 	*pGroupSelect ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_SetMemberListArray", NULL ) ;

	/* Selected gourp */

	pGroupSelect = XawListShowCurrent ( wDlgMemoSendListGroup ) ;

	if ( pGroupSelect == NULL ) {
		iGroupSelect = 0 ;
	} else if ( pGroupSelect->list_index < 0 ) {
		iGroupSelect = 0 ;
	} else {
		iGroupSelect = pGroupSelect->list_index ;
	}

	/* Free old list */

	if ( pListArray != NULL ) {
		XtFree ( (void*) pListArray ) ;
	}

	if ( pMemberArray != NULL ) {
		XtFree ( (void*) pMemberArray ) ;
	}

	/* Count effective entry */

	iListCount = 0 ;
	for ( pChain = zMemberAnchor.pHead ; pChain != NULL ; pChain = pChain->pNext ) {

		pMember = (zMemberType*) ( pChain->pData ) ;

		if ( ! pMember->bDelete ) iListCount ++ ;

	}

	/* Allocate memory */

	pListArray = (char**) XtCalloc ( sizeof(char*), iListCount + 1 ) ;
	if ( pListArray == NULL ) {
		xmemo_ErrorLog ( "xmemo_SetMemberListArray", "XtCalloc", NULL ) ;
		return ;
	}

	pMemberArray = (zMemberType**) XtCalloc ( sizeof(zMemberType*), iListCount + 1 ) ;
	if ( pMemberArray == NULL ) {
		xmemo_ErrorLog ( "xmemo_SetMemberListArray", "XtCalloc", NULL ) ;
		return ;
	}

	/* Set list entry ( Sorted by IP address ) */

	iListCount = 0 ;
	for ( pChain = zMemberAnchor.pHead ; pChain != NULL ; pChain = pChain->pNext ) {

		pMember = (zMemberType*) ( pChain->pData ) ;

		if ( pMember->bDelete ) continue ;

		if ( iGroupSelect >= 1 ) {
			if ( pMember->pGroup == NULL ) continue ;
			if ( strcmp ( pGroupArray[iGroupSelect], pMember->pGroup ) != 0 ) continue ;
		}

		if ( pMember->pLine == NULL ) xmemo_SetMemberListLine ( pMember ) ;
		if ( pMember->pLine == NULL ) continue ;
		for ( i = 0 ; i < iListCount ; i ++ ) {
			if ( pMemberArray[i]->iAddr >  pMember->iAddr
			  || pMemberArray[i]->iAddr == pMember->iAddr
			  && pMemberArray[i]->iPort >  pMember->iPort ) break ;
		}
		for ( j = iListCount - 1 ; j >= i ; j -- ) {
			pListArray[j+1] = pListArray[j] ;
			pMemberArray[j+1] = pMemberArray[j] ;
		}
		pListArray[i] = pMember->pLine ;
		pMemberArray[i] = pMember ;
		iListCount ++ ;

	}
	pListArray[iListCount] = NULL ;

	/* Refresh list widget */

	n = 0 ;
	XtSetArg ( xArg[n], XtNlist, pListArray ) ; n ++ ;
	XtSetArg ( xArg[n], XtNnumberStrings, iListCount ) ; n ++ ;
	XtSetValues ( wDlgMemoSendListUser, xArg, n ) ;
	XawListChange ( wDlgMemoSendListUser, pListArray, iListCount, 0, False ) ;
	iListSelect = -1 ;

}

/******************************************************************************/
/*  xmemo_RefreshMemberList                                                   */
/******************************************************************************/

static void xmemo_RefreshMemberList ( caddr_t pClientData, XtIntervalId* pId )
{

time_t	tInterval ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_RefreshMemberList", NULL ) ;

	iRefreshTimer += 2 ;

	if ( iRefreshTimer >= 8 ) {
		bRefreshTimer = FALSE ;
		return ;
	}

	if ( iRefreshTimer == 2 ) {
		xmemo_SetGroupListArray ( ) ;
		xmemo_SetMemberListArray ( ) ;
		xmemo_NetNotify_Broadcast ( XMEMO_NET_NOTIFY_ENTER ) ;
	}

	tInterval = 2000 ;
	XtAppAddTimeOut ( xApp, tInterval, (XtTimerCallbackProc) xmemo_RefreshMemberList, NULL ) ;

}

/******************************************************************************/
/*  SendRetain_Click                                                          */
/******************************************************************************/

static void SendRetain_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

zMemoType		*pMemo ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "SendRetain_Click", NULL ) ;

	XtPopdown ( wDlgMemoSend ) ;

	xmemo_NetSend_MemberList ( pMemoActive ) ;

	if ( pMemoActive->pChain == NULL ) {
		xmemo_InsertMemo ( pMemoActive ) ;
	}

}

static XtCallbackRec pSendRetain_CallbackRec [ ] =
{
	{ (XtCallbackProc) SendRetain_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  SendRemove_Click                                                          */
/******************************************************************************/

static void SendRemove_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

zMemoType		*pMemo ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "SendRemove_Click", NULL ) ;

	XtPopdown ( wDlgMemoSend ) ;

	xmemo_NetSend_MemberList ( pMemoActive ) ;

	if ( pMemoActive->pChain == NULL ) {
		xmemo_FreeMemo ( pMemoActive ) ;
	} else {
		xmemo_DeleteMemo ( pMemoActive ) ;
	}

}

static XtCallbackRec pSendRemove_CallbackRec [ ] =
{
	{ (XtCallbackProc) SendRemove_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  SendCancel_Click                                                          */
/******************************************************************************/

static void SendCancel_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

zMemoType		*pMemo ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "SendCancel_Click", NULL ) ;

	XtPopdown ( wDlgMemoSend ) ;

	if ( pMemoActive->pChain == NULL ) {
		xmemo_FreeMemo ( pMemoActive ) ;
	}

}

static XtCallbackRec pSendCancel_CallbackRec [ ] =
{
	{ (XtCallbackProc) SendCancel_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  SendSearch_Click                                                          */
/******************************************************************************/

static void SendSearch_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

time_t	tInterval ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "SendSearch_Click", NULL ) ;

	if ( ! bRefreshTimer ) {
		xmemo_FreeMemberAll ( ) ;
		xmemo_SetGroupListArray ( ) ;
		xmemo_SetMemberListArray ( ) ;
	}

	xmemo_NetNotify_Broadcast ( XMEMO_NET_NOTIFY_ENTER ) ;

	if ( bRefreshTimer ) return ;

	bRefreshTimer = TRUE ;
	iRefreshTimer = 0 ;

	tInterval = 2000 ;
	XtAppAddTimeOut ( xApp, tInterval, (XtTimerCallbackProc) xmemo_RefreshMemberList, NULL ) ;

}

static XtCallbackRec pSendSearch_CallbackRec [ ] =
{
	{ (XtCallbackProc) SendSearch_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  SendListGroup_Click                                                       */
/******************************************************************************/

static void SendListGroup_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

char	sLog [ 16 ] ;
XawListReturnStruct	*pListClick ;

	pListClick = (XawListReturnStruct*) pCallData ;

	if ( bMemoTraceLog ) {
		sprintf ( sLog, "%d", pListClick->list_index ) ;
		xmemo_TraceLog ( "SendListGroup_Click", sLog ) ;
	}

	/* Refresh list widget */

	xmemo_SetMemberListArray ( ) ;

}

static XtCallbackRec pSendListGroup_CallbackRec [ ] =
{
	{ (XtCallbackProc) SendListGroup_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  SendListUser_Click                                                        */
/******************************************************************************/

static void SendListUser_Click ( Widget w, caddr_t pClientData, caddr_t pCallData )
{

char	sLog [ 16 ] ;
XawListReturnStruct	*pListClick ;
int 	n ;
Arg 	xArg [ 8 ] ;
zMemberType	*pMember ;

	pListClick = (XawListReturnStruct*) pCallData ;

	if ( bMemoTraceLog ) {
		sprintf ( sLog, "%d", pListClick->list_index ) ;
		xmemo_TraceLog ( "SendListUser_Click", sLog ) ;
	}

	XawListUnhighlight ( wDlgMemoSendListUser ) ;

	pMember = pMemberArray[pListClick->list_index] ;
	pMember->bSelect = ! pMember->bSelect ;

	if ( pMember->bSelect ) *pListArray[pListClick->list_index] = '*' ;
	else                    *pListArray[pListClick->list_index] = ' ' ;

	/* Refresh list widget */

	n = 0 ;
	XtSetArg ( xArg[n], XtNlist, pListArray ) ; n ++ ;
	XtSetArg ( xArg[n], XtNnumberStrings, iListCount ) ; n ++ ;
	XtSetValues ( wDlgMemoSendListUser, xArg, n ) ;
	XawListChange ( wDlgMemoSendListUser, pListArray, iListCount, 0, False ) ;
	iListSelect = -1 ;

}

static XtCallbackRec pSendListUser_CallbackRec [ ] =
{
	{ (XtCallbackProc) SendListUser_Click, NULL },
	{ NULL, NULL }
} ;

/******************************************************************************/
/*  xmemo_InitDialog_Send                                                    */
/******************************************************************************/

int xmemo_InitDialog_Send ( )
{

static	Arg 	xArgViewportUser [ ] =
{
	{ XtNwidth     , 400 },
	{ XtNheight    , 400 },
} ;

int 	n ;
Arg 	xArg [ 16 ] ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_InitDialog_Send", NULL ) ;

	/* Dialog shell */

	n = 0 ;
	XtSetArg ( xArg[n], XtNtitle , "xmemo - Send"           ) ; n ++ ;
	XtSetArg ( xArg[n], XtNwidth , iDefaultSendDialogWidth  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNheight, iDefaultSendDialogHeight ) ; n ++ ;
	wDlgMemoSend = XtCreatePopupShell ( "DlgSend", transientShellWidgetClass, wTop, xArg, n ) ;
	                                  
	/* Main form */

	wDlgMemoSendForm = XtCreateManagedWidget ( "Form", formWidgetClass, wDlgMemoSend, xArg, 0 ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNtop   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom, XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft  , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright , XtRubber      ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, NULL   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNallowHoriz, False ) ; n ++ ;
	XtSetArg ( xArg[n], XtNallowVert , True  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNuseBottom , True  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNuseRight  , True  ) ; n ++ ;
	wDlgMemoSendViewportGroup = XtCreateManagedWidget ( "ListViewport", viewportWidgetClass, wDlgMemoSendForm, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNwidth , 100 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNheight, 200 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom, XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft  , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright , XtChainRight  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNdefaultColumns, 1     ) ; n ++ ;
	XtSetArg ( xArg[n], XtNforceColumns  , TRUE  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback, pSendListGroup_CallbackRec ) ; n ++ ;
	wDlgMemoSendListGroup = XtCreateManagedWidget ( "List", listWidgetClass, wDlgMemoSendViewportGroup, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNtop   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom, XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft  , XtRubber      ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright , XtChainRight  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, wDlgMemoSendViewportGroup ) ; n ++ ;
	XtSetArg ( xArg[n], XtNallowHoriz, False ) ; n ++ ;
	XtSetArg ( xArg[n], XtNallowVert , True  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNuseBottom , True  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNuseRight  , True  ) ; n ++ ;
	wDlgMemoSendViewportUser = XtCreateManagedWidget ( "ListViewport", viewportWidgetClass, wDlgMemoSendForm, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNwidth , 400 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNheight, 200 ) ; n ++ ;
	XtSetArg ( xArg[n], XtNtop   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom, XtChainBottom ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft  , XtChainLeft   ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright , XtChainRight  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNdefaultColumns, 1     ) ; n ++ ;
	XtSetArg ( xArg[n], XtNforceColumns  , TRUE  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback, pSendListUser_CallbackRec ) ; n ++ ;
	wDlgMemoSendListUser = XtCreateManagedWidget ( "List", listWidgetClass, wDlgMemoSendViewportUser, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNtop   , XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNbottom, XtChainTop    ) ; n ++ ;
	XtSetArg ( xArg[n], XtNleft  , XtChainRight  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNright , XtChainRight  ) ; n ++ ;
	XtSetArg ( xArg[n], XtNfromHoriz, wDlgMemoSendViewportUser ) ; n ++ ;
	wDlgMemoSendMenu = XtCreateManagedWidget ( "MenuBar", boxWidgetClass, wDlgMemoSendForm, xArg, n ) ;

	/* Menu button */

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel   , "Send" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback, pSendRetain_CallbackRec ) ; n ++ ;
	XtCreateManagedWidget ( "CmdButton", commandWidgetClass, wDlgMemoSendMenu, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel   , "Send & Remove" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback, pSendRemove_CallbackRec ) ; n ++ ;
	XtCreateManagedWidget ( "CmdButton", commandWidgetClass, wDlgMemoSendMenu, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel   , "Cancel" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback, pSendCancel_CallbackRec ) ; n ++ ;
	XtCreateManagedWidget ( "CmdButton", commandWidgetClass, wDlgMemoSendMenu, xArg, n ) ;

	n = 0 ;
	XtSetArg ( xArg[n], XtNlabel   , "Search" ) ; n ++ ;
	XtSetArg ( xArg[n], XtNcallback, pSendSearch_CallbackRec ) ; n ++ ;
	XtCreateManagedWidget ( "CmdButton", commandWidgetClass, wDlgMemoSendMenu, xArg, n ) ;

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_PopupDialog_Send                                                    */
/******************************************************************************/

int xmemo_PopupDialog_Send ( )
{

zChainType	*pChain ;
zMemberType	*pMember ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_PopupDialog_Send", NULL ) ;

	for ( pChain = zMemberAnchor.pHead ; pChain != NULL ; pChain = pChain->pNext ) {
		pMember = (zMemberType*) pChain->pData ;
		pMember->bSelect = FALSE ;
		if ( pMember->pLine != NULL ) *(pMember->pLine) = ' ' ;
	}

	xmemo_SetGroupListArray ( ) ;
	xmemo_SetMemberListArray ( ) ;

	XtPopup ( wDlgMemoSend, XtGrabExclusive ) ;

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_PopdownDialog_Send                                                  */
/******************************************************************************/

int xmemo_PopdownDialog_Send ( )
{

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_PopdownDialog_Send", NULL ) ;

	XtPopdown ( wDlgMemoSend ) ;

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_FreeMemberAll                                                       */
/******************************************************************************/

int xmemo_FreeMemberAll ( )
{

zMemberType		*pMember ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_FreeMemberAll", NULL ) ;

	while ( zMemberAnchor.pHead != NULL ) {

		pMember = (zMemberType*) ( zMemberAnchor.pHead->pData ) ;

		/* Remove from chained list */

		xmemo_RemoveFromChain ( &zMemberAnchor, zMemberAnchor.pHead ) ;

		/* Free allocated memory */

		xmemo_FreeMember ( pMember ) ;

	}

}

/******************************************************************************/
/*  xmemo_CreateMember                                                        */
/******************************************************************************/

int xmemo_CreateMember ( zMemberType *pMember )
{

zChainType	*pChain ;
zMemberType *pOldMember, *pNewMember ;
int 	iLen ;

	for ( pChain = zMemberAnchor.pHead ; pChain != NULL ; pChain = pChain->pNext ) {

		pOldMember = (zMemberType*) ( pChain->pData ) ;

		if ( pMember->cProtocol == pOldMember->cProtocol
		  && pMember->iAddr     == pOldMember->iAddr
		  && pMember->iPort     == pOldMember->iPort ) pOldMember->bDelete = TRUE ;

	}

	/* Allocate memory */

	pNewMember = (zMemberType*) XtCalloc ( sizeof(zMemberType), 1 ) ;
	if ( pNewMember == NULL ) {
		xmemo_ErrorLog ( "xmemo_CreateMember", "XtCalloc", NULL ) ;
		return RC_ERROR ;
	}

	/* Set values */

	pNewMember->cProtocol = pMember->cProtocol ;
	pNewMember->iAddr     = pMember->iAddr ;
	pNewMember->iPort     = pMember->iPort ;
	pNewMember->bAbsence  = pMember->bAbsence ;
	pNewMember->bDelete   = FALSE ;

	if ( pMember->pHost != NULL ) {
		iLen = strlen ( pMember->pHost ) + 1 ;
		if ( iLen > MAXLEN_HOSTNAME ) iLen = MAXLEN_HOSTNAME ;
		pNewMember->pHost = XtMalloc ( iLen ) ;
		if ( pNewMember->pHost == NULL ) {
			xmemo_ErrorLog ( "OptionNetworkAdd_Click", "XtMalloc", NULL ) ;
			xmemo_FreeMember ( pNewMember ) ;
			return ;
		}
		strncpy ( pNewMember->pHost, pMember->pHost, iLen ) ;
		pNewMember->pHost[iLen-1] = 0 ;
	}

	if ( pMember->pGroup != NULL ) {
		iLen = strlen ( pMember->pGroup ) + 1 ;
		if ( iLen > MAXLEN_GROUPNAME ) iLen = MAXLEN_GROUPNAME ;
		pNewMember->pGroup = XtMalloc ( iLen ) ;
		if ( pNewMember->pGroup == NULL ) {
			xmemo_ErrorLog ( "OptionNetworkAdd_Click", "XtMalloc", NULL ) ;
			xmemo_FreeMember ( pNewMember ) ;
			return ;
		}
		strncpy ( pNewMember->pGroup, pMember->pGroup, iLen ) ;
		pNewMember->pGroup[iLen-1] = 0 ;
	}

	if ( pMember->pNick != NULL ) {
		iLen = strlen ( pMember->pNick ) + 1 ;
		if ( iLen > MAXLEN_NICKNAME ) iLen = MAXLEN_NICKNAME ;
		pNewMember->pNick = XtMalloc ( iLen ) ;
		if ( pNewMember->pNick == NULL ) {
			xmemo_ErrorLog ( "OptionNetworkAdd_Click", "XtMalloc", NULL ) ;
			xmemo_FreeMember ( pNewMember ) ;
			return ;
		}
		strncpy ( pNewMember->pNick, pMember->pNick, iLen ) ;
		pNewMember->pNick[iLen-1] = 0 ;
	}

	if ( pMember->pUser != NULL ) {
		iLen = strlen ( pMember->pUser ) + 1 ;
		if ( iLen > MAXLEN_USERNAME ) iLen = MAXLEN_USERNAME ;
		pNewMember->pUser = XtMalloc ( iLen ) ;
		if ( pNewMember->pUser == NULL ) {
			xmemo_ErrorLog ( "OptionNetworkAdd_Click", "XtMalloc", NULL ) ;
			xmemo_FreeMember ( pNewMember ) ;
			return ;
		}
		strncpy ( pNewMember->pUser, pMember->pUser, iLen ) ;
		pNewMember->pUser[iLen-1] = 0 ;
	}

	/* Insert into chained list */

	pNewMember->pChain = xmemo_InsertIntoChain ( &zMemberAnchor, pNewMember ) ;
	if ( pNewMember->pChain == NULL ) {
		xmemo_ErrorLog ( "xmemo_CreateMember", "xmemo_InsertIntoChain", NULL ) ;
		xmemo_FreeMember ( pNewMember ) ;
		return ;
	}

	/* Refresh list */

	if ( XtIsRealized ( wDlgMemoSend )  &&  ( ! bRefreshTimer  ||  iRefreshTimer >= 2 ) ) {
		xmemo_SetGroupListArray ( ) ;
		xmemo_SetMemberListArray ( ) ;
	}

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_DeleteMember                                                        */
/******************************************************************************/

int xmemo_DeleteMember ( zMemberType *pMember )
{

zChainType	*pChain ;
zMemberType *pOldMember ;

	for ( pChain = zMemberAnchor.pHead ; pChain != NULL ; pChain = pChain->pNext ) {

		pOldMember = (zMemberType*) ( pChain->pData ) ;

		if ( pMember->cProtocol == pOldMember->cProtocol
		  && pMember->iAddr     == pOldMember->iAddr
		  && pMember->iPort     == pOldMember->iPort ) pOldMember->bDelete = TRUE ;

	}

	return RC_SUCCESS ;

}
