/******************************************************************************/
/*                                                                            */
/*  <<< Copyright >>>                                                         */
/*                                                                            */
/*  The "xmemo" software is copyright 1999 by Takao Abe.                      */
/*  All rights reserved.                                                      */
/*                                                                            */
/*  The "xmemo" software refers to the source code, the binary code,          */
/*  the documents, and all derivatives based on this software under           */
/*  the copyright law, that is a software containing a program or a           */
/*  portion of a program, regardless of modifications and/or translations.    */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  <<< Licence and redistribution >>>                                        */
/*                                                                            */
/*  The source code and the binary code are free software.                    */
/*  The use and redistribution of source code and binary code are permitted   */
/*  provided that the following conditions are met:                           */
/*  You may use the software for private use and commercial use,              */
/*  and may modify your copy or copies of the software or any portion of it.  */
/*                                                                            */
/*  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESSED OR IMPLIED            */
/*  WARRANTIES OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED        */
/*  WARRANTIES OF MERCHANTABLILITY AND FITNESS FOR A PARTICULAR PURPOSE.      */
/*  IN NO EVENT SHALL THE AUTHOR TAKAO ABE BE LIABLE FOR ANY DIRECT,          */
/*  INDIRECT, GENERAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES           */
/*  ( INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR       */
/*  SERVICES; LOSS OF USE, DATA OR PROFITS; OR BUSINESS INTERRUPTION )        */
/*  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,       */
/*  STRICT LIABILITY, OR TORT ( INCLUDING NEGLIGENCE OR OTHERWISE )           */
/*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED       */
/*  OF THE POSSIBILITY OF SUCH DAMAGE.                                        */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with or without     */
/*  modification, the whole program or a portion of it, are permitted         */
/*  provided that one retains the entire copyright notice properly, and       */
/*  displays or retains this license in the documentation and/or other        */
/*  materials provided with the distribution.                                 */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with modification,  */
/*  the whole program or a portion of it, are permitted provided that one     */
/*  declares that it is modified software and not original, and displays or   */
/*  describes any modifications in the documentation and/or other materials   */
/*  provided with the distribution.                                           */
/*                                                                            */
/*  This software and the name of the author must not be used to endorse or   */
/*  promote products derived from this software without prior written         */
/*  permission.                                                               */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  EMail   abetakao@bea.hi-ho.ne.jp                                          */
/*          HCA00763@nifty.ne.jp                                              */
/*                                                                            */
/*  URL     http://www.bea.hi-ho.ne.jp/abetakao/                              */
/*                                                                            */
/******************************************************************************/


#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<time.h>

#include	<X11/Intrinsic.h>

#include	"xmemo.h"
#include	"xmemodata.h"
#include	"xmemofunc.h"


/******************************************************************************/
/*  Local variables                                                           */
/******************************************************************************/


static	int 	bParam_z ;	/* Trace */


/******************************************************************************/
/*  main                                                                      */
/******************************************************************************/

main ( int argc, char *argv[ ] )
{

int 	rc ;
zMemoType		*pMemo ;
zChainType		*pChain ;
XEvent  		xEvt ;

	rc = xmemo_Param ( argc, argv ) ;
/*
	if ( rc == RC_ERROR ) {
		printf ( "xmemo : Invalid parameter\n" ) ;
		exit ( 1 ) ;
	}
*/

	rc= xmemo_Init ( ) ;
	if ( rc == RC_ERROR ) {
		printf ( "xmemo : Initialize failure\n" ) ;
		exit ( 1 ) ;
	}

	xmemo_InitWindow ( argc, argv ) ;

	xmemo_InitNetwork ( ) ;

	XtAppMainLoop ( xApp ) ;

}

/******************************************************************************/
/*  xmemo_Init                                                                */
/******************************************************************************/

int xmemo_Init ( )
{

int 	rc ;
char	sLog [ 32 + MAXLEN_FILENAME ] ;
char	*pStr ;

	/* Initialize */

	pVersion   = "xmemo     Version 1.01" ;
	pCopyright = "Copyright (C) 1999, Takao Abe.  All rights reserved." ;

	bMemoTraceLog = bParam_z ;
	bMemoTracePacket = bParam_z ;

	zMemoAnchor.pHead = zMemoAnchor.pTail = NULL ;
	zMemberAnchor.pHead = zMemberAnchor.pTail = NULL ;
	zNetworkAnchor.pHead = zNetworkAnchor.pTail = NULL ;

	/* Presence,Absence */

	bUserAbsence = FALSE ;

	/* User information */

	sMemoPassword[0] = 0 ;
	sMemoUsername[0] = 0 ;
	sMemoNickname[0] = 0 ;
	sMemoGroupname[0] = 0 ;

	pStr = getenv ( "USER" ) ;
	if ( pStr == NULL ) pStr = getenv ( "LOGNAME" ) ;
	if ( pStr != NULL ) {
		strncpy ( sMemoUsername, pStr, MAXLEN_USERNAME ) ;
	} else {
		strncpy ( sMemoUsername, "", MAXLEN_USERNAME ) ;
	}
	sMemoUsername[MAXLEN_USERNAME-1] = 0 ;

	/* System default value ( Charset ) */

	cMemoCharset = CHARSET_EUC ;

	/* System default values ( Directory ) */

	pStr = getenv ( "HOME" ) ;
	if ( pStr != NULL ) strcpy ( sMemoHomeDir, pStr ) ;
	else                strcpy ( sMemoHomeDir, "." ) ;
	strcpy ( sMemoBaseDir, sMemoHomeDir ) ;
	strcat ( sMemoBaseDir, "/" ) ;
	strcat ( sMemoBaseDir, "xmemo.box" ) ;
	strcpy ( sMemoDataDir, "data" ) ;
	strcpy ( sMemoConfDir, "conf" ) ;
	strcpy ( sMemoLogDir , "log" ) ;

	/* System default values ( File names ) */

	strcpy ( sFilenameOptionProfile, "profile" ) ;
	if ( strlen ( sMemoUsername ) > 0 ) {
		strcat  ( sFilenameOptionProfile, "." ) ;
		strncat ( sFilenameOptionProfile, sMemoUsername, MAXLEN_FILENAME - strlen ( sFilenameOptionProfile ) ) ;
		sFilenameOptionProfile[MAXLEN_FILENAME-1] = 0 ;
	}
	strcpy ( sFilenameOptionPreference, "default.ini" ) ;
	strcpy ( sFilenameOptionNetwork   , "network.ini" ) ;
	strcpy ( sFilenameOptionLog       , "log.ini" ) ;
	strcpy ( sFilenameOptionSystem    , "system.ini" ) ;

	strcpy ( sFilenameMemoArchives    , "xmemo.dat" ) ;
	strcpy ( sFilenameMemoTransactions, "transactions.dat" ) ;

	strcpy ( sFilenameErrorLog, "error.log" ) ;
	strcpy ( sFilenameTraceLog, "trace.log" ) ;

	/* Preference */

	iDefaultMemoWidth  = 120 ;
	iDefaultMemoHeight = 60 ;
	iDefaultMemoBackColor[0] = iDefaultMemoBackColor[1] = iDefaultMemoBackColor[2] = 59392 ;
	iDefaultMemoBorderWidth = 2 ;
	iDefaultMemoBorderColor[0] = iDefaultMemoBorderColor[1] = iDefaultMemoBorderColor[2] = 0 ;
	iDefaultMemoFontColor[0] = iDefaultMemoFontColor[1] = iDefaultMemoFontColor[2] = 0 ;
	cDefaultMemoIconPosition = XMEMO_ICON_POSITION_LEFT_TOP ;
	iDefaultMemoIconType = 0 ;

	/* Program profile ( ~/.xmemorc ) */

	xmemo_Init_LoadProfile ( ) ;

	if ( bMemoTraceLog ) {
		xmemo_TraceLog ( pVersion, NULL ) ;
		xmemo_TraceLog ( "xmemo_Init", sMemoHomeDir ) ;
		xmemo_TraceLog ( "xmemo_Init", sMemoBaseDir ) ;
	}

	/* Create directory */

	rc = xmemo_CreateDirectory ( ) ;
	if ( rc == RC_ERROR ) return rc ;

	/* Load options */

	xmemo_LoadOptionPreference ( ) ;

	xmemo_LoadOptionProfile ( ) ;

	xmemo_LoadOptionLog ( ) ;

	xmemo_LoadOptionNetwork ( ) ;

	xmemo_LoadOptionSystem ( ) ;

	/* Load memo data */
	
	xmemo_LoadMemoAll ( ) ;

	/* Initialize completed */

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_Init_LoadProfile                                                    */
/******************************************************************************/

int xmemo_Init_LoadProfile ( )
{

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_Exit                                                                */
/******************************************************************************/

int xmemo_Exit ( )
{

	/* Close network */

	xmemo_ExitNetwork ( ) ;

	/* Free member data */

	xmemo_FreeMemberAll ( ) ;

	/* Save memo data */

	xmemo_SaveMemoAll ( ) ;

	/* Free memo data */

	xmemo_FreeMemoAll ( ) ;

}

/******************************************************************************/
/*  xmemo_Param                                                               */
/******************************************************************************/

int xmemo_Param ( int argc, char *argv[ ] )
{

int 	i ;
int 	rc = RC_SUCCESS ;

	bParam_z = FALSE ;

	/* Parse command line parameters */
	for ( i = 1 ; i < argc ; i ++ ) {
		if ( argv[i][0] == '-' ) {
			switch ( argv[i][1] ) {
			case 'z' :
				if ( argv[i][2] == 0 ) {
					bParam_z = TRUE ;
				} else {
					rc = RC_ERROR ;
				}
				break ;
			default :
				rc = RC_ERROR ;
				break ;
			}
		} else {
			rc = RC_ERROR ;
		}
	}

	return rc ;

}
