/******************************************************************************/
/*                                                                            */
/*  <<< Copyright >>>                                                         */
/*                                                                            */
/*  The "xmemo" software is copyright 1999 by Takao Abe.                      */
/*  All rights reserved.                                                      */
/*                                                                            */
/*  The "xmemo" software refers to the source code, the binary code,          */
/*  the documents, and all derivatives based on this software under           */
/*  the copyright law, that is a software containing a program or a           */
/*  portion of a program, regardless of modifications and/or translations.    */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  <<< Licence and redistribution >>>                                        */
/*                                                                            */
/*  The source code and the binary code are free software.                    */
/*  The use and redistribution of source code and binary code are permitted   */
/*  provided that the following conditions are met:                           */
/*  You may use the software for private use and commercial use,              */
/*  and may modify your copy or copies of the software or any portion of it.  */
/*                                                                            */
/*  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESSED OR IMPLIED            */
/*  WARRANTIES OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED        */
/*  WARRANTIES OF MERCHANTABLILITY AND FITNESS FOR A PARTICULAR PURPOSE.      */
/*  IN NO EVENT SHALL THE AUTHOR TAKAO ABE BE LIABLE FOR ANY DIRECT,          */
/*  INDIRECT, GENERAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES           */
/*  ( INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR       */
/*  SERVICES; LOSS OF USE, DATA OR PROFITS; OR BUSINESS INTERRUPTION )        */
/*  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,       */
/*  STRICT LIABILITY, OR TORT ( INCLUDING NEGLIGENCE OR OTHERWISE )           */
/*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED       */
/*  OF THE POSSIBILITY OF SUCH DAMAGE.                                        */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with or without     */
/*  modification, the whole program or a portion of it, are permitted         */
/*  provided that one retains the entire copyright notice properly, and       */
/*  displays or retains this license in the documentation and/or other        */
/*  materials provided with the distribution.                                 */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with modification,  */
/*  the whole program or a portion of it, are permitted provided that one     */
/*  declares that it is modified software and not original, and displays or   */
/*  describes any modifications in the documentation and/or other materials   */
/*  provided with the distribution.                                           */
/*                                                                            */
/*  This software and the name of the author must not be used to endorse or   */
/*  promote products derived from this software without prior written         */
/*  permission.                                                               */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  EMail   abetakao@bea.hi-ho.ne.jp                                          */
/*          HCA00763@nifty.ne.jp                                              */
/*                                                                            */
/*  URL     http://www.bea.hi-ho.ne.jp/abetakao/                              */
/*                                                                            */
/******************************************************************************/


#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<time.h>

#include	<X11/Intrinsic.h>

#include	"xmemo.h"
#include	"xmemodata.h"
#include	"xmemofunc.h"


/******************************************************************************/
/*  xmemo_LoadOptionNetwork                                                   */
/******************************************************************************/

int xmemo_LoadOptionNetwork ( )
{

char	sLog [ 100 ] ;
char	fn [ MAXLEN_FILENAME ] ;
FILE	*fp ;
int 	i, n, rc ;

char	sBuf [ MAXLEN_HOSTNAME + 100 ] ;
char	*pCh ;
char	*pToken [ 6 ] ;
zNetworkType	*pNetwork ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_LoadOptionNetwork", NULL ) ;

	/* Read file */

	strcpy ( fn, sMemoBaseDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sMemoConfDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sFilenameOptionNetwork ) ;
	fp = fopen ( fn, "r" ) ;
	if ( fp == NULL ) {
		return RC_SUCCESS ;
	}

	rc = RC_SUCCESS ;

	for ( i = 0 ; ; i ++ ) {

		if ( fgets ( sBuf, sizeof(sBuf), fp ) == NULL ) break ;
		pCh = strchr ( sBuf, '\n' ) ;
		if ( pCh != NULL ) *pCh = 0 ;

		/* Allocate memory */

		pNetwork = (zNetworkType*) XtCalloc ( sizeof(zNetworkType), 1 ) ;
		if ( pNetwork == NULL ) {
			xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "XtCalloc", NULL ) ;
			break ;
		}

		n = xmemo_CsvToken ( pToken, 6, sBuf ) ;
		if ( n < 5  &&  6 < n ) {
			sprintf ( sLog, "File=[%s]  Line=%d", sFilenameOptionNetwork, i ) ;
			xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "Invalid line", sLog ) ;
			continue ;
		}

		/* Enable,Disable */
		if ( strlen ( pToken[0] ) != 1 ) {
			sprintf ( sLog, "File=[%s]  Line=%d  Field=1", sFilenameOptionNetwork, i ) ;
			xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "Invalid line", sLog ) ;
			continue ;
		}
		switch ( *pToken[0] ) {
		case '1' :
			pNetwork->bEnable = TRUE ;
			break ;
		case '0' :
			pNetwork->bEnable = FALSE ;
			break ;
		default :
			sprintf ( sLog, "File=[%s]  Line=%d  Field=1", sFilenameOptionNetwork, i ) ;
			xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "Invalid line", sLog ) ;
			continue ;
		}

		/* Protocol */
		if ( strlen ( pToken[1] ) != 1 ) {
			sprintf ( sLog, "File=[%s]  Line=%d  Field=2", sFilenameOptionNetwork, i ) ;
			xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "Invalid line", sLog ) ;
			continue ;
		}
		switch ( *pToken[1] ) {
		case XMEMO_PROTOCOL_IPMSG    :
		case XMEMO_PROTOCOL_OBOEGAKI :
		case XMEMO_PROTOCOL_HUSEN    :
			pNetwork->cProtocol = *pToken[1] ;
			break ;
		default :
			sprintf ( sLog, "File=[%s]  Line=%d  Field=2", sFilenameOptionNetwork, i ) ;
			xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "Invalid line", sLog ) ;
			continue ;
		}

		/* Port */
		if ( strlen ( pToken[2] ) == 0 ) {
			pNetwork->pPort = NULL ;
		} else if ( xmemo_CheckNumericRange ( pToken[2], 1, 65535 ) ) {
			pToken[2] = strtok ( pToken[2], " " ) ;
			pNetwork->pPort = (char*) XtMalloc ( strlen ( pToken[2] ) + 1 ) ;
			if ( pNetwork == NULL ) {
				xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "XtMalloc", NULL ) ;
				rc = RC_ERROR ;
				break ;
			}
			strcpy ( pNetwork->pPort, pToken[2] ) ;
			pNetwork->iPort = atoi ( pNetwork->pPort ) ;
		} else {
			sprintf ( sLog, "File=[%s]  Line=%d  Field=3", sFilenameOptionNetwork, i ) ;
			xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "Invalid line", sLog ) ;
			continue ;
		}

		/* Search */
		if ( strlen ( pToken[3] ) != 1 ) {
			sprintf ( sLog, "File=[%s]  Line=%d  Field=4", sFilenameOptionNetwork, i ) ;
			xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "Invalid line", sLog ) ;
			continue ;
		}
		switch ( *pToken[3] ) {
		case XMEMO_SEARCH_BROADCAST :
		case XMEMO_SEARCH_HOST      :
		case XMEMO_SEARCH_RANGE     :
			pNetwork->cSearch = *pToken[3] ;
			break ;
		default :
			sprintf ( sLog, "File=[%s]  Line=%d  Field=4", sFilenameOptionNetwork, i ) ;
			xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "Invalid line", sLog ) ;
			continue ;
		}

		/* Address */
		if ( strlen ( pToken[4] ) == 0 ) {
			sprintf ( sLog, "File=[%s]  Line=%d  Field=5", sFilenameOptionNetwork, i ) ;
			xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "Invalid line", sLog ) ;
			continue ;
		} else {
			pToken[4] = strtok ( pToken[4], " " ) ;
			pNetwork->pAddr[0] = (char*) XtMalloc ( strlen ( pToken[4] ) + 1 ) ;
			if ( pNetwork->pAddr[0] == NULL ) {
				xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "XtMalloc", NULL ) ;
				rc = RC_ERROR ;
				break ;
			}
			strcpy ( pNetwork->pAddr[0], pToken[4] ) ;
			pNetwork->iAddr[0] = ipaddress_string_to_long ( pNetwork->pAddr[0] ) ;
		}

		if ( pToken[5] == NULL ) {
			pNetwork->pAddr[1] = NULL ;
		} else if ( strlen ( pToken[5] ) == 0 ) {
			pNetwork->pAddr[1] = NULL ;
		} else {
			pToken[5] = strtok ( pToken[5], " " ) ;
			pNetwork->pAddr[1] = (char*) XtMalloc ( strlen ( pToken[5] ) + 1 ) ;
			if ( pNetwork->pAddr[1] == NULL ) {
				xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "XtMalloc", NULL ) ;
				rc = RC_ERROR ;
				break ;
			}
			strcpy ( pNetwork->pAddr[1], pToken[5] ) ;
			pNetwork->iAddr[1] = ipaddress_string_to_long ( pNetwork->pAddr[1] ) ;
		}

		/* Insert into chained list */

		pNetwork->pChain = xmemo_InsertIntoChain ( &zNetworkAnchor, pNetwork) ;
		if ( pNetwork->pChain == NULL ) {
			xmemo_ErrorLog ( "xmemo_LoadOptionNetwork", "xmemo_InsertIntoChain", NULL ) ;
			XtFree ( (void*) pNetwork) ;
			rc = RC_ERROR ;
			break ;
		}

	}

	fclose ( fp ) ;

	return rc ;

}

/******************************************************************************/
/*  xmemo_SaveOptionNetwork                                                   */
/******************************************************************************/

int xmemo_SaveOptionNetwork ( )
{

char	sLog [ 100 ] ;
char	fn [ MAXLEN_FILENAME ] ;
FILE	*fp ;
int 	rc ;

zChainType	*pChain ;
zNetworkType	*pNetwork;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_SaveOptionNetwork", NULL ) ;

	sprintf ( sLog, "File=[%s]", sFilenameOptionNetwork ) ;

	strcpy ( fn, sMemoBaseDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sMemoConfDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sFilenameOptionNetwork ) ;
	fp = fopen ( fn, "w" ) ;
	if ( fp == NULL ) {
		xmemo_ErrorLog ( "xmemo_SaveOptionNetwork", "fopen", sLog ) ;
		return RC_ERROR ;
	}

	for ( pChain = zNetworkAnchor.pHead ; pChain != NULL ; pChain = pChain->pNext ) {

		pNetwork = (zNetworkType*) ( pChain->pData ) ;

		if ( pNetwork->bEnable ) rc = fputs ( "1,", fp ) ;
		else                     rc = fputs ( "0,", fp ) ;
		if ( rc == EOF ) break ;

		if ( pNetwork->pPort != NULL ) rc = fprintf ( fp, "%c,%s,", pNetwork->cProtocol, pNetwork->pPort ) ;
		else                           rc = fprintf ( fp, "%c,%s,", pNetwork->cProtocol, "" ) ;
		if ( rc == EOF ) break ;

		if ( pNetwork->pAddr[0] != NULL ) rc = fprintf ( fp, "%c,%s,", pNetwork->cSearch, pNetwork->pAddr[0] ) ;
		else                              rc = fprintf ( fp, "%c,%s,", pNetwork->cSearch, "" ) ;
		if ( rc == EOF ) break ;

		if ( pNetwork->pAddr[1] != NULL ) rc = fprintf ( fp, "%s\n", pNetwork->pAddr[1] ) ;
		else                              rc = fprintf ( fp, "%s\n", "" ) ;
		if ( rc == EOF ) break ;

	}

	fclose ( fp ) ;

	if ( rc == EOF ) {
		xmemo_ErrorLog ( "xmemo_SaveOptionNetwork", "fprintf", sLog ) ;
		return RC_ERROR ;
	}

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_LoadOptionPreference                                                */
/******************************************************************************/

int xmemo_LoadOptionPreference ( )
{

char	sLog [ 50 + MAXLEN_FILENAME ] ;
char	fn [ MAXLEN_FILENAME ] ;
FILE	*fp ;
int 	i, n, rc ;

char	sBuf [ 100 ] ;
char	*pCh ;
char	*pToken [ 6 ] ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_LoadOptionPreference", NULL ) ;

	/* Read file */

	strcpy ( fn, sMemoBaseDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sMemoConfDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sFilenameOptionPreference ) ;
	fp = fopen ( fn, "r" ) ;
	if ( fp == NULL ) {
		return RC_SUCCESS ;
	}

	/* File format ID */
	if ( fgets ( sBuf, sizeof(sBuf), fp ) == NULL ) {
		fclose ( fp ) ;
		return RC_ERROR ;
	}
	pCh = strchr ( sBuf, '\n' ) ;
	if ( pCh != NULL ) *pCh = 0 ;

	rc = RC_SUCCESS ;

	for ( i = 2 ; ; i ++ ) {

		if ( fgets ( sBuf, sizeof(sBuf), fp ) == NULL ) break ;
		pCh = strchr ( sBuf, '\n' ) ;
		if ( pCh != NULL ) *pCh = 0 ;

		switch ( i ) {

		case 2 : /* Width, Height */
			n = xmemo_CsvToken ( pToken, 2, sBuf ) ;
			if ( n == 2
			  && xmemo_CheckNumericRange ( pToken[0], MIN_DEFAULT_MEMO_WIDTH , MAX_DEFAULT_MEMO_WIDTH  ) 
			  && xmemo_CheckNumericRange ( pToken[1], MIN_DEFAULT_MEMO_HEIGHT, MAX_DEFAULT_MEMO_HEIGHT ) ) {
				iDefaultMemoWidth  = atoi ( pToken[0] ) ;
				iDefaultMemoHeight = atoi ( pToken[1] ) ;
			} else {
				sprintf ( sLog, "File=[%s]  Line=%d", sFilenameOptionPreference, i ) ;
				xmemo_ErrorLog ( "xmemo_LoadOptionPreference", "Invalid", sLog ) ;
				rc = RC_ERROR ;
			}
			break ;

		case 3 : /* Background color */
			n = xmemo_CsvToken ( pToken, 3, sBuf ) ;
			if ( n == 3
			  && xmemo_CheckNumericRange ( pToken[0], 0, 65535 ) 
			  && xmemo_CheckNumericRange ( pToken[1], 0, 65535 ) 
			  && xmemo_CheckNumericRange ( pToken[2], 0, 65535 ) ) {
				iDefaultMemoBackColor[0] = atoi ( pToken[0] ) ; 		/* Red   */
				iDefaultMemoBackColor[1] = atoi ( pToken[1] ) ; 		/* Green */
				iDefaultMemoBackColor[2] = atoi ( pToken[2] ) ; 		/* Blue  */
			} else {
				sprintf ( sLog, "File=[%s]  Line=%d", sFilenameOptionPreference, i ) ;
				xmemo_ErrorLog ( "xmemo_LoadOptionPreference", "Invalid", sLog ) ;
				rc = RC_ERROR ;
			}
			break ;

		case 4 : /* Border width */
			if ( xmemo_CheckNumericRange ( sBuf, MIN_DEFAULT_BORDER_WIDTH, MAX_DEFAULT_BORDER_WIDTH ) ) {
				iDefaultMemoBorderWidth = atoi ( sBuf ) ;
			} else {
				sprintf ( sLog, "File=[%s]  Line=%d", sFilenameOptionPreference, i ) ;
				xmemo_ErrorLog ( "xmemo_LoadOptionPreference", "Invalid", sLog ) ;
				rc = RC_ERROR ;
			}
			break ;

		case 5 : /* Border color */
			n = xmemo_CsvToken ( pToken, 3, sBuf ) ;
			if ( n == 3
			  && xmemo_CheckNumericRange ( pToken[0], 0, 65535 ) 
			  && xmemo_CheckNumericRange ( pToken[1], 0, 65535 ) 
			  && xmemo_CheckNumericRange ( pToken[2], 0, 65535 ) ) {
				iDefaultMemoBorderColor[0] = atoi ( pToken[0] ) ;		/* Red   */
				iDefaultMemoBorderColor[1] = atoi ( pToken[1] ) ;		/* Green */
				iDefaultMemoBorderColor[2] = atoi ( pToken[2] ) ;		/* Blue  */
			} else {
				sprintf ( sLog, "File=[%s]  Line=%d", sFilenameOptionPreference, i ) ;
				xmemo_ErrorLog ( "xmemo_LoadOptionPreference", "Invalid", sLog ) ;
				rc = RC_ERROR ;
			}
			break ;

		case 6 : /* Font name */
			if ( strlen ( sBuf ) < MAXLEN_FONTNAME ) {
				strncpy ( sDefaultMemoFontName, sBuf, MAXLEN_FONTNAME ) ;
				sDefaultMemoFontName[MAXLEN_FONTNAME-1] = 0 ;
			} else {
				sprintf ( sLog, "File=[%s]  Line=%d", sFilenameOptionPreference, i ) ;
				xmemo_ErrorLog ( "xmemo_LoadOptionPreference", "Invalid", sLog ) ;
				rc = RC_ERROR ;
			}
			break ;

		case 7 : /* Font color */
			n = xmemo_CsvToken ( pToken, 3, sBuf ) ;
			if ( n == 3
			  && xmemo_CheckNumericRange ( pToken[0], 0, 65535 ) 
			  && xmemo_CheckNumericRange ( pToken[1], 0, 65535 ) 
			  && xmemo_CheckNumericRange ( pToken[2], 0, 65535 ) ) {
				iDefaultMemoFontColor[0] = atoi ( pToken[0] ) ; 		/* Red   */
				iDefaultMemoFontColor[1] = atoi ( pToken[1] ) ; 		/* Green */
				iDefaultMemoFontColor[2] = atoi ( pToken[2] ) ; 		/* Blue  */
			} else {
				sprintf ( sLog, "File=[%s]  Line=%d", sFilenameOptionPreference, i ) ;
				xmemo_ErrorLog ( "xmemo_LoadOptionPreference", "Invalid", sLog ) ;
				rc = RC_ERROR ;
			}
			break ;

		case 8 : /* Icon position, Icon type */
			n = xmemo_CsvToken ( pToken, 2, sBuf ) ;
			if ( n == 2
			  && xmemo_CheckNumeric ( pToken[0] ) 
			  && xmemo_CheckNumeric ( pToken[1] ) ) {
				cDefaultMemoIconPosition = atoi ( pToken[0] ) ;
				iDefaultMemoIconType     = atoi ( pToken[1] ) ;
			} else {
				sprintf ( sLog, "File=[%s]  Line=%d", sFilenameOptionPreference, i ) ;
				xmemo_ErrorLog ( "xmemo_LoadOptionPreference", "Invalid", sLog ) ;
				rc = RC_ERROR ;
			}
			break ;

		default : 
			break ;

		}

	}

	fclose ( fp ) ;

	return rc ;

}

/******************************************************************************/
/*  xmemo_SaveOptionPreference                                                */
/******************************************************************************/

int xmemo_SaveOptionPreference ( )
{

char	sLog [ 50 + MAXLEN_FILENAME ] ;
char	fn [ MAXLEN_FILENAME ] ;
FILE	*fp ;
int 	i, rc ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_SaveOptionPreference", NULL ) ;

	sprintf ( sLog, "File=[%s]", sFilenameOptionPreference ) ;

	strcpy ( fn, sMemoBaseDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sMemoConfDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sFilenameOptionPreference ) ;
	fp = fopen ( fn, "w" ) ;
	if ( fp == NULL ) {
		xmemo_ErrorLog ( "xmemo_SaveOptionPreference", "fopen", sLog ) ;
		return RC_ERROR ;
	}

	for ( i = 1 ; i <= 7 ; i ++ ) {

		switch ( i ) {

		case 1 : /* File format ID */
			rc = fputs ( "1\n", fp ) ;
			break ;

		case 2 : /* Width, Height */
			rc = fprintf ( fp, "%d,%d\n", iDefaultMemoWidth, iDefaultMemoHeight ) ;
			break ;

		case 3 : /* Background color */
			rc = fprintf ( fp, "%d,%d,%d\n", iDefaultMemoBackColor[0], iDefaultMemoBackColor[1], iDefaultMemoBackColor[2] ) ;
			break ;

		case 4 : /* Border width */
			rc = fprintf ( fp, "%d\n", iDefaultMemoBorderWidth ) ;
			break ;

		case 5 : /* Border color */
			rc = fprintf ( fp, "%d,%d,%d\n", iDefaultMemoBorderColor[0], iDefaultMemoBorderColor[1], iDefaultMemoBorderColor[2] ) ;
			break ;

		case 6 : /* Font name */
			rc = fprintf ( fp, "%s\n", sDefaultMemoFontName ) ;
			break ;

		case 7 : /* Font color */
			rc = fprintf ( fp, "%d,%d,%d\n", iDefaultMemoFontColor[0], iDefaultMemoFontColor[1], iDefaultMemoFontColor[2] ) ;
			break ;

		case 8 : /* Icon position, Icon type */
			rc = fprintf ( fp, "%d,%d\n", (int) cDefaultMemoIconPosition, (int) iDefaultMemoIconType ) ;
			break ;

		default :
			break ;

		}

		if ( rc == EOF ) {
			xmemo_ErrorLog ( "xmemo_SaveOptionPreference", "fprintf", sLog ) ;
			fclose ( fp ) ;
			return RC_ERROR ;
		}

	}

	fclose ( fp ) ;

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_LoadOptionLog                                                       */
/******************************************************************************/

int xmemo_LoadOptionLog ( )
{

char	sLog [ 50 + MAXLEN_FILENAME ] ;
char	fn [ MAXLEN_FILENAME ] ;
FILE	*fp ;
int 	i, n, rc ;

char	sBuf [ 100 ] ;
char	*pCh ;
char	*pToken [ 6 ] ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_LoadOptionLog", NULL ) ;

	/* Read file */

	strcpy ( fn, sMemoBaseDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sMemoConfDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sFilenameOptionLog ) ;
	fp = fopen ( fn, "r" ) ;
	if ( fp == NULL ) {
		return RC_SUCCESS ;
	}

	/* File format ID */
	if ( fgets ( sBuf, sizeof(sBuf), fp ) == NULL ) {
		fclose ( fp ) ;
		return RC_ERROR ;
	}
	pCh = strchr ( sBuf, '\n' ) ;
	if ( pCh != NULL ) *pCh = 0 ;

	rc = RC_SUCCESS ;

	for ( i = 2 ; ; i ++ ) {

		if ( fgets ( sBuf, sizeof(sBuf), fp ) == NULL ) break ;
		pCh = strchr ( sBuf, '\n' ) ;
		if ( pCh != NULL ) *pCh = 0 ;

		switch ( i ) {

		case 2 : /* Log flags */
			n = xmemo_CsvToken ( pToken, 3, sBuf ) ;
			if ( n == 3
			  && xmemo_CheckNumericRange ( pToken[0], 0, 1 ) 
			  && xmemo_CheckNumericRange ( pToken[1], 0, 1 ) 
			  && xmemo_CheckNumericRange ( pToken[2], 0, 1 ) ) {
				bMemoLogSend    = ( *pToken[0] == '1' ) ;
				bMemoLogReceive = ( *pToken[1] == '1' ) ;
				bMemoLogDelete  = ( *pToken[2] == '1' ) ;
			} else {
				sprintf ( sLog, "File=[%s]  Line=%d", sFilenameOptionPreference, i ) ;
				xmemo_ErrorLog ( "xmemo_LoadOptionLog", "Invalid", sLog ) ;
				rc = RC_ERROR ;
			}
			break ;

		case 3 : /* Keep */
			if ( xmemo_CheckNumericRange ( sBuf, 0, 99999 ) ) {
				iMemoLogKeep = atoi ( sBuf ) ;
			} else {
				sprintf ( sLog, "File=[%s]  Line=%d", sFilenameOptionPreference, i ) ;
				xmemo_ErrorLog ( "xmemo_LoadOptionLog", "Invalid", sLog ) ;
				rc = RC_ERROR ;
			}
			break ;

		default :
			break ;

		}

	}

	fclose ( fp ) ;

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_SaveOptionLog                                                       */
/******************************************************************************/

int xmemo_SaveOptionLog ( )
{

char	sLog [ 50 + MAXLEN_FILENAME ] ;
char	fn [ MAXLEN_FILENAME ] ;
FILE	*fp ;
int 	i, rc ;
int 	iFlag [ 3 ] ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_SaveOptionLog", NULL ) ;

	sprintf ( sLog, "File=[%s]", sFilenameOptionLog ) ;

	strcpy ( fn, sMemoBaseDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sMemoConfDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sFilenameOptionLog ) ;
	fp = fopen ( fn, "w" ) ;
	if ( fp == NULL ) {
		xmemo_ErrorLog ( "xmemo_SaveOptionLog", "fopen", sLog ) ;
		return RC_ERROR ;
	}

	for ( i = 1 ; i <= 3 ; i ++ ) {

		switch ( i ) {

		case 1 : /* File format ID */
			rc = fputs ( "1\n", fp ) ;
			break ;

		case 2 : /* Log flags */
			if ( bMemoLogSend    ) iFlag[0] = 1 ;
			else                   iFlag[0] = 0 ;
			if ( bMemoLogReceive ) iFlag[1] = 1 ;
			else                   iFlag[1] = 0 ;
			if ( bMemoLogDelete  ) iFlag[2] = 1 ;
			else                   iFlag[2] = 0 ;
			rc = fprintf ( fp, "%d,%d,%d\n", iFlag[0], iFlag[1], iFlag[2] ) ;
			break ;

		case 3 : /* Keep */
			rc = fprintf ( fp, "%d\n", iMemoLogKeep ) ;
			break ;

		default :
			break ;

		}

		if ( rc == EOF ) {
			xmemo_ErrorLog ( "xmemo_SaveOptionLog", "fprintf", sLog ) ;
			fclose ( fp ) ;
			return RC_ERROR ;
		}

	}

	fclose ( fp ) ;

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_LoadOptionProfile                                                   */
/******************************************************************************/

int xmemo_LoadOptionProfile ( )
{

char	*pStr ;
char	fn [ MAXLEN_FILENAME ] ;
FILE	*fp ;
int 	i, bEndFile, rc ;
char	sBuf [ MAXLEN_USERNAME + MAXLEN_PASSWORD + MAXLEN_NICKNAME + 6 ] ;
char	*pCh ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_LoadOptionProfile", NULL ) ;

	/* Read file */

	if ( strlen(sMemoConfDir) + 1 + strlen(sFilenameOptionProfile) + strlen(sMemoUsername) + 1 > MAXLEN_FILENAME ) {
		xmemo_ErrorLog ( "xmemo_LoadOptionProfile", "File name length", NULL ) ;
		return RC_ERROR ;
	}

	strcpy ( fn, sMemoBaseDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sMemoConfDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sFilenameOptionProfile ) ;
	fp = fopen ( fn, "r" ) ;
	if ( fp == NULL ) {
		return RC_SUCCESS ;
	}

	/* File format ID */
	if ( fgets ( sBuf, sizeof(sBuf), fp ) == NULL ) {
		fclose ( fp ) ;
		return RC_ERROR ;
	}
	pCh = strchr ( sBuf, '\n' ) ;
	if ( pCh != NULL ) *pCh = 0 ;

	rc = RC_SUCCESS ;
	bEndFile = FALSE ;
	for ( i = 2 ; ! bEndFile ; i ++ ) {

		if ( fgets ( sBuf, sizeof(sBuf), fp ) == NULL ) break ;
		pCh = strchr ( sBuf, '\n' ) ;
		if ( pCh != NULL ) *pCh = 0 ;

		switch ( i ) {

		case 2 : /* Username */
			if ( strncmp ( sMemoUsername, sBuf, MAXLEN_USERNAME ) != 0 ) {
				xmemo_ErrorLog ( "xmemo_LoadOptionProfile", "Invalid file", fn ) ;
				rc = RC_ERROR ;
				bEndFile = TRUE ;
			}
			break ;

		case 3 : /* Password */
			strncpy ( sMemoPassword, sBuf, MAXLEN_PASSWORD ) ;
			sMemoPassword[MAXLEN_NICKNAME-1] = 0 ;
			break ;

		case 4 : /* Nickname */
			strncpy ( sMemoNickname, sBuf, MAXLEN_NICKNAME ) ;
			sMemoNickname[MAXLEN_NICKNAME-1] = 0 ;
			break ;

		case 5 : /* Groupname */
			strncpy ( sMemoGroupname, sBuf, MAXLEN_GROUPNAME ) ;
			sMemoGroupname[MAXLEN_GROUPNAME-1] = 0 ;
			break ;

		default :
			bEndFile = TRUE ;
			break ;

		}

	}

	fclose ( fp ) ;

	return rc ;

}

/******************************************************************************/
/*  xmemo_SaveOptionProfile                                                   */
/******************************************************************************/

int xmemo_SaveOptionProfile ( )
{

char	sLog [ MAXLEN_FILENAME + 100 ] ;
char	fn [ MAXLEN_FILENAME ] ;
FILE	*fp ;
int 	i, bEndFile, rc ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_SaveOptionProfile", NULL ) ;

	/* Write file */

	if ( strlen(sMemoConfDir) + 1 + strlen(sFilenameOptionProfile) + strlen(sMemoUsername) + 1 > MAXLEN_FILENAME ) {
		xmemo_ErrorLog ( "xmemo_SaveOptionProfile", "File name length", NULL ) ;
		return RC_ERROR ;
	}

	strcpy ( fn, sMemoBaseDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sMemoConfDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sFilenameOptionProfile ) ;
	fp = fopen ( fn, "w" ) ;
	if ( fp == NULL ) {
		xmemo_ErrorLog ( "xmemo_SaveOptionProfile", "fopen", sLog ) ;
		return RC_ERROR ;
	}

	rc = RC_SUCCESS ;
	bEndFile = FALSE ;
	for ( i = 1 ; ( ! bEndFile ) && rc == RC_SUCCESS ; i ++ ) {

		switch ( i ) {

		case 1 : /* File format ID */
			if ( fputs ( "1", fp ) == EOF ) rc = RC_ERROR ;
			break ;

		case 2 : /* Username */
			if ( fputs ( sMemoUsername, fp ) == EOF ) rc = RC_ERROR ;
			break ;

		case 3 : /* Password */
			if ( fputs ( sMemoPassword, fp ) == EOF ) rc = RC_ERROR ;
			break ;

		case 4 : /* Nickname */
			if ( fputs ( sMemoNickname, fp ) == EOF ) rc = RC_ERROR ;
			break ;

		case 5 : /* Groupname */
			if ( fputs ( sMemoGroupname, fp ) == EOF ) rc = RC_ERROR ;
			break ;

		default :
			bEndFile = TRUE ;
			break ;

		}

		if ( ! bEndFile ) {
			if ( fputc ( '\n', fp ) == EOF ) rc = RC_ERROR ;
		}

	}

	fclose ( fp ) ;

	if ( rc == RC_ERROR ) {
		sprintf ( sLog, "File=[%s]   Line=%d", sFilenameOptionProfile, i ) ;
		xmemo_ErrorLog ( "xmemo_SaveOptionProfile", "fputs", sLog ) ;
		return RC_ERROR ;
	}

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_LoadOptionSystem                                                    */
/******************************************************************************/

int xmemo_LoadOptionSystem ( )
{

char	sLog [ 50 + MAXLEN_FILENAME ] ;
char	fn [ MAXLEN_FILENAME ] ;
FILE	*fp ;
int 	i, n, rc ;

char	sBuf [ 100 ] ;
char	*pCh ;
char	*pToken [ 5 ] ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_LoadOptionSystem", NULL ) ;

	/* Read file */

	strcpy ( fn, sMemoBaseDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sMemoConfDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sFilenameOptionSystem ) ;
	fp = fopen ( fn, "r" ) ;
	if ( fp == NULL ) {
		return RC_SUCCESS ;
	}

	/* File format ID */
	if ( fgets ( sBuf, sizeof(sBuf), fp ) == NULL ) {
		fclose ( fp ) ;
		return RC_ERROR ;
	}
	pCh = strchr ( sBuf, '\n' ) ;
	if ( pCh != NULL ) *pCh = 0 ;

	rc = RC_SUCCESS ;

	for ( i = 2 ; ; i ++ ) {

		if ( fgets ( sBuf, sizeof(sBuf), fp ) == NULL ) break ;
		pCh = strchr ( sBuf, '\n' ) ;
		if ( pCh != NULL ) *pCh = 0 ;

		switch ( i ) {

		case 2 : /* Character set */
			n = xmemo_CsvToken ( pToken, 1, sBuf ) ;
			if ( n == 1 ) pCh = strtok ( pToken[0], " " ) ;
			if ( n == 1  &&  pCh != NULL ) {
				switch ( *pCh ) {
				case CHARSET_ASCII :
				case CHARSET_EUC   :
				case CHARSET_JIS7  :
				case CHARSET_JIS8  :
				case CHARSET_SJIS  :
					cMemoCharset = *pCh ;
					break ;
				default : 
					sprintf ( sLog, "File=[%s]  Line=%d", sFilenameOptionSystem, i ) ;
					xmemo_ErrorLog ( "xmemo_LoadOptionSystem", "Invalid", sLog ) ;
					rc = RC_ERROR ;
					break ;
				}
			} else {
				sprintf ( sLog, "File=[%s]  Line=%d", sFilenameOptionSystem, i ) ;
				xmemo_ErrorLog ( "xmemo_LoadOptionSystem", "Invalid", sLog ) ;
				rc = RC_ERROR ;
			}
			break ;

		default :
			break ;

		}

	}

	fclose ( fp ) ;

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_SaveOptionSystem                                                    */
/******************************************************************************/

int xmemo_SaveOptionSystem ( )
{

char	sLog [ 50 + MAXLEN_FILENAME ] ;
char	fn [ MAXLEN_FILENAME ] ;
FILE	*fp ;
int 	i, rc ;
int 	iFlag [ 3 ] ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_SaveOptionSystem", NULL ) ;

	sprintf ( sLog, "File=[%s]", xmemo_SaveOptionSystem ) ;

	strcpy ( fn, sMemoBaseDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sMemoConfDir ) ;
	strcat ( fn, "/" ) ;
	strcat ( fn, sFilenameOptionSystem ) ;
	fp = fopen ( fn, "w" ) ;
	if ( fp == NULL ) {
		xmemo_ErrorLog ( "xmemo_SaveOptionSystem", "fopen", sLog ) ;
		return RC_ERROR ;
	}

	for ( i = 1 ; i <= 2 ; i ++ ) {

		switch ( i ) {

		case 1 : /* File format ID */
			rc = fputs ( "1\n", fp ) ;
			break ;

		case 2 : /* Character set */
			rc = fprintf ( fp, "%c\n", cMemoCharset ) ;
			break ;

		default :
			break ;

		}

		if ( rc == EOF ) {
			xmemo_ErrorLog ( "xmemo_SaveOptionSystem", "fprintf", sLog ) ;
			fclose ( fp ) ;
			return RC_ERROR ;
		}


	}

	fclose ( fp ) ;

	return RC_SUCCESS ;

}
