/******************************************************************************/
/*                                                                            */
/*  <<< Copyright >>>                                                         */
/*                                                                            */
/*  The "xmemo" software is copyright 1999 by Takao Abe.                      */
/*  All rights reserved.                                                      */
/*                                                                            */
/*  The "xmemo" software refers to the source code, the binary code,          */
/*  the documents, and all derivatives based on this software under           */
/*  the copyright law, that is a software containing a program or a           */
/*  portion of a program, regardless of modifications and/or translations.    */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  <<< Licence and redistribution >>>                                        */
/*                                                                            */
/*  The source code and the binary code are free software.                    */
/*  The use and redistribution of source code and binary code are permitted   */
/*  provided that the following conditions are met:                           */
/*  You may use the software for private use and commercial use,              */
/*  and may modify your copy or copies of the software or any portion of it.  */
/*                                                                            */
/*  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESSED OR IMPLIED            */
/*  WARRANTIES OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED        */
/*  WARRANTIES OF MERCHANTABLILITY AND FITNESS FOR A PARTICULAR PURPOSE.      */
/*  IN NO EVENT SHALL THE AUTHOR TAKAO ABE BE LIABLE FOR ANY DIRECT,          */
/*  INDIRECT, GENERAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES           */
/*  ( INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR       */
/*  SERVICES; LOSS OF USE, DATA OR PROFITS; OR BUSINESS INTERRUPTION )        */
/*  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,       */
/*  STRICT LIABILITY, OR TORT ( INCLUDING NEGLIGENCE OR OTHERWISE )           */
/*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED       */
/*  OF THE POSSIBILITY OF SUCH DAMAGE.                                        */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with or without     */
/*  modification, the whole program or a portion of it, are permitted         */
/*  provided that one retains the entire copyright notice properly, and       */
/*  displays or retains this license in the documentation and/or other        */
/*  materials provided with the distribution.                                 */
/*                                                                            */
/*  Redistribution and use in source form or binary form, with modification,  */
/*  the whole program or a portion of it, are permitted provided that one     */
/*  declares that it is modified software and not original, and displays or   */
/*  describes any modifications in the documentation and/or other materials   */
/*  provided with the distribution.                                           */
/*                                                                            */
/*  This software and the name of the author must not be used to endorse or   */
/*  promote products derived from this software without prior written         */
/*  permission.                                                               */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  EMail   abetakao@bea.hi-ho.ne.jp                                          */
/*          HCA00763@nifty.ne.jp                                              */
/*                                                                            */
/*  URL     http://www.bea.hi-ho.ne.jp/abetakao/                              */
/*                                                                            */
/******************************************************************************/


#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<time.h>
#include	<errno.h>
#include	<sys/types.h>
#include	<sys/time.h>
#include	<sys/ioctl.h>
#include	<sys/socket.h>
#ifdef __sparc
#include	<sys/filio.h>
#endif
#include	<netdb.h>
#include	<netinet/in.h>
#include	<arpa/inet.h>


#include	<X11/Intrinsic.h>


#include	"xmemo.h"
#include	"xmemodata.h"
#include	"xmemofunc.h"


/******************************************************************************/
/*  Local constant definition                                                 */
/******************************************************************************/

#define		XMEMO_TIMEOUT_CONNECT	10
#define		XMEMO_TIMEOUT_RECEIVE	5

/******************************************************************************/
/*  xmemo_RecvUdp                                                             */
/******************************************************************************/

static void xmemo_RecvUdp ( caddr_t pClientData, int *pSock, XtInputId *pId )
{

zSocketType	*pSocket ;
int 	rc ;
char	data [ MAXLEN_PACKET + 4 ] ;
long	iPeerAddr ;
short	iPeerPort ;
struct	sockaddr_in	addr ;
#if defined(_AIX) || defined(_nec)
size_t	addrlen ;
#else
int 	addrlen ;
#endif

	pSocket = (zSocketType*) pClientData ;

	memset ( &addr, 0, sizeof(addr) ) ;
	addr.sin_family      = AF_INET ;
	addr.sin_port        = htons ( pSocket->iPort ) ;
	addr.sin_addr.s_addr = htonl ( pSocket->iAddr ) ;
	addrlen = sizeof(addr) ;
	rc = recvfrom ( pSocket->iSock, data, MAXLEN_PACKET, 0, (struct sockaddr *)&addr, &addrlen ) ;
	if ( rc < 0 ) {
		xmemo_ErrorLog ( "xmemo_RecvUdp", "recvfrom", NULL ) ;
	}	
	data[rc] = 0 ;

	iPeerAddr = ntohl ( addr.sin_addr.s_addr ) ;
	iPeerPort = ntohs ( addr.sin_port ) ;

	if ( bMemoTraceLog ) {
		char	sLog [ 32 ] ;
		sprintf ( sLog, "%c://%s:%u",
			pSocket->cProtocol,
			ipaddress_long_to_string ( iPeerAddr ),
			pSocket->iPort ) ;
		xmemo_TraceLog ( "xmemo_RecvUdp", sLog ) ;
	}
	if ( bMemoTracePacket ) xmemo_TracePacket ( data, rc ) ;

	switch ( pSocket->cProtocol ) {
	case XMEMO_PROTOCOL_HUSEN :
		xmemo_RecvUdp_Husen    ( pSocket, iPeerAddr, iPeerPort, data, rc ) ; break ;
	case XMEMO_PROTOCOL_IPMSG :
		xmemo_RecvUdp_Ipmsg    ( pSocket, iPeerAddr, iPeerPort, data, rc ) ; break ;
	case XMEMO_PROTOCOL_OBOEGAKI :
		xmemo_RecvUdp_Oboegaki ( pSocket, iPeerAddr, iPeerPort, data, rc ) ; break ;
	}

}

/******************************************************************************/
/*  xmemo_RecvAcceptedTcp                                                     */
/******************************************************************************/

#if 0
void xmemo_RecvAcceptedTcp ( caddr_t pClientData, int *pSock, XtInputId *pId )
{

int 	i, rc ;
char	data [ MAXLEN_PACKET + 4 ] ;
zSocketType	*pSocket ;

	pSocket = (zSocketType*) pClientData ;

	rc = recv ( pSocket->iSock, data, MAXLEN_PACKET, 0 ) ;
	if ( rc < 0 ) {
		xmemo_ErrorLog ( "xmemo_RecvAcceptedTcp", "recv", NULL ) ;
	}	
	data[rc] = 0 ;

	if ( bMemoTraceLog ) {
		char	sLog [ 32 ] ;
		sprintf ( sLog, "%c://%s:%u",
			pSocket->cProtocol,
			ipaddress_long_to_string ( pSocket->iAddr ),
			pSocket->iPort ) ;
		xmemo_TraceLog ( "xmemo_RecvAcceptedTcp", sLog ) ;
	}
	if ( bMemoTracePacket ) xmemo_TracePacket ( data, rc ) ;

	switch ( pSocket->cProtocol ) {
	case XMEMO_PROTOCOL_HUSEN :
		break ;
	case XMEMO_PROTOCOL_IPMSG :
		break ;
	case XMEMO_PROTOCOL_OBOEGAKI :
		xmemo_RecvAcceptedTcp_Oboegaki ( pSocket, data, rc ) ; break ;
	}

}
#endif

/******************************************************************************/
/*  xmemo_RecvConnectedTcp                                                    */
/******************************************************************************/

static void xmemo_RecvConnectedTcp ( caddr_t pClientData, int *pSock, XtInputId *pId )
{

int 	i, rc ;
char	data [ MAXLEN_PACKET + 4 ] ;
zMemberType	*pSendPeer ;
zSocketType	*pSocket ;

	pSendPeer = (zMemberType*) pClientData ;
	pSocket = pSendPeer->pTcpSocket ;

	rc = recv ( pSocket->iSock, data, MAXLEN_PACKET, 0 ) ;
	if ( rc < 0 ) {
		xmemo_ErrorLog ( "xmemo_RecvConnectedTcp", "recv", NULL ) ;
	}	
	data[rc] = 0 ;

	if ( bMemoTraceLog ) {
		char	sLog [ 32 ] ;
		sprintf ( sLog, "%c://%s:%u",
			pSocket->cProtocol,
			ipaddress_long_to_string ( pSocket->iAddr ),
			pSocket->iPort ) ;
		xmemo_TraceLog ( "xmemo_RecvConnectedTcp", sLog ) ;
	}
	if ( bMemoTracePacket ) xmemo_TracePacket ( data, rc ) ;

	switch ( pSocket->cProtocol ) {
	case XMEMO_PROTOCOL_HUSEN :
		break ;
	case XMEMO_PROTOCOL_IPMSG :
		break ;
	case XMEMO_PROTOCOL_OBOEGAKI :
		xmemo_RecvConnectedTcp_Oboegaki ( pSendPeer, data, rc ) ; break ;
	}

}

/******************************************************************************/
/*  xmemo_AcceptTcp                                                           */
/******************************************************************************/

static void xmemo_AcceptTcp ( caddr_t pClientData, int *pSock, XtInputId *pId )
{

zSocketType	*pSocket ;
int 	iClientSocket ;
struct	sockaddr_in	addr ;
#if defined(_AIX) || defined(_nec)
size_t	addrlen ;
#else
int 	addrlen ;
#endif

	pSocket = (zSocketType*) pClientData ;

	memset ( &addr, 0, sizeof(addr) ) ;
#if 0
	addr.sin_family      = AF_INET ;
	addr.sin_port        = htons ( pSocket->iPort ) ;
	addr.sin_addr.s_addr = htonl ( pSocket->iAddr ) ;
#endif
	addrlen = sizeof(addr) ;
	iClientSocket = accept ( pSocket->iSock, (struct sockaddr *)&addr, &addrlen ) ;
	if ( iClientSocket < 0 ) {
		xmemo_ErrorLog ( "xmemo_AcceptTcp", "accept", NULL ) ;
	}	

	if ( bMemoTraceLog ) {
		char	sLog [ 32 ] ;
		sprintf ( sLog, "%c://%s:%u",
			pSocket->cProtocol,
			ipaddress_long_to_string ( ntohl ( addr.sin_addr.s_addr ) ),
			pSocket->iPort ) ;
		xmemo_TraceLog ( "xmemo_AcceptTcp", sLog ) ;
	}

	switch ( pSocket->cProtocol ) {
	case XMEMO_PROTOCOL_HUSEN :
		close ( iClientSocket ) ; break ;
	case XMEMO_PROTOCOL_IPMSG :
		close ( iClientSocket ) ; break ;
	case XMEMO_PROTOCOL_OBOEGAKI :
		xmemo_AcceptTcp_Oboegaki ( pSocket, &addr, (int)addrlen, iClientSocket ) ; break ;
	}

}

/******************************************************************************/
/*  xmemo_ConnectTcp                                                          */
/******************************************************************************/

static void xmemo_ConnectTcp ( caddr_t pClientData, int *pSock, XtInputId *pId )
{

zMemberType	*pMember ;
zSocketType	*pSocket ;

	pMember = (zMemberType*) pClientData ;
	pSocket = pMember->pTcpSocket ;

	if ( bMemoTraceLog ) {
		char	sLog [ 32 ] ;
		sprintf ( sLog, "%c://%s:%u",
			pSocket->cProtocol,
			ipaddress_long_to_string ( pSocket->iAddr ),
			pSocket->iPort ) ;
		xmemo_TraceLog ( "xmemo_ConnectTcp", sLog ) ;
	}

	pMember->iState = XMEMO_STATE_ANY_CONNECTED ;

	/* Remove callback */

	XtRemoveInput ( *pId ) ;
	pSocket->iCallbackWriteId = 0 ;

	/* Do nothing */

	switch ( pSocket->cProtocol ) {
	case XMEMO_PROTOCOL_HUSEN :
		break ;
	case XMEMO_PROTOCOL_IPMSG :
		break ;
	case XMEMO_PROTOCOL_OBOEGAKI :
		time ( &(pMember->tTimeout) ) ;
		pMember->tTimeout += XMEMO_TIMEOUT_RECEIVE ;
		break ;
	}

}

/******************************************************************************/
/*  xmemo_CreateUdpSocket                                                     */
/******************************************************************************/

static zSocketType* xmemo_CreateUdpSocket ( zNetworkType *pNetwork )
{

char	sLog [ 64 ] ;
zSocketType	*pSocket ;
int 	rc ;
struct	sockaddr_in	addr ;
#ifdef __sparc
char	bFlag ;
#else
int 	bFlag ;
#endif

	if ( bMemoTraceLog ) {
		sprintf ( sLog, "%c://%s:%u",
			pNetwork->cProtocol,
			ipaddress_long_to_string ( pNetwork->iAddr[0] ),
			(unsigned short) pNetwork->iPort ) ;
		if ( pNetwork->cSearch == XMEMO_SEARCH_RANGE ) {
			strcat ( sLog, " - " ) ;
			strcat ( sLog, ipaddress_long_to_string ( pNetwork->iAddr[1] ) ) ;
		}
		xmemo_TraceLog ( "xmemo_CreateUdpSocket", sLog ) ;
	}

	/* Allocate memory */

	pSocket = (zSocketType*) XtMalloc ( sizeof(zSocketType) ) ;
	if ( pSocket == NULL ) {
		xmemo_ErrorLog ( "xmemo_CreateUdpSocket", "XtMalloc", NULL ) ;
		return NULL ;
	}

	pSocket->cSockType = XMEMO_SOCKET_UDP ;
	pSocket->cProtocol = pNetwork->cProtocol ;
	pSocket->iAddr = ntohl ( INADDR_ANY ) ;
	pSocket->iPort = pNetwork->iPort ;
	pSocket->tTimeout = 0 ;
	pSocket->iCallbackReadId   = 0 ;
	pSocket->iCallbackWriteId  = 0 ;
	pSocket->iCallbackExceptId = 0 ;

	/* Socket */

	pSocket->iSock = socket ( AF_INET, SOCK_DGRAM, 0 ) ;
	if ( pSocket->iSock == 0 ) {
		xmemo_ErrorLog ( "xmemo_CreateUdpSocket", "socket", NULL ) ;
		XtFree ( (void*) pSocket ) ;
		return NULL ;
	}

	/* Bind */

	memset ( &addr, 0, sizeof(addr) ) ;
	addr.sin_family      = AF_INET ;
	addr.sin_port        = htons ( pSocket->iPort ) ;
	addr.sin_addr.s_addr = htonl ( pSocket->iAddr ) ;
	rc = bind ( pSocket->iSock, (struct sockaddr *)&addr, sizeof(addr) ) ;
	if ( rc != 0 ) {
		xmemo_ErrorLog ( "xmemo_CreateUdpSocket", "bind", NULL ) ;
		close ( pSocket->iSock ) ;
		XtFree ( (void*) pSocket ) ;
		return NULL ;
	}

	bFlag = 1 ;
	rc = setsockopt ( pSocket->iSock, SOL_SOCKET, SO_BROADCAST, &bFlag, sizeof(bFlag) ) ;
	if ( rc != 0 ) {
		xmemo_ErrorLog ( "xmemo_CreateUdpSocket", "setsocket", NULL ) ;
		close ( pSocket->iSock ) ;
		XtFree ( (void*) pSocket ) ;
		return NULL ;
	}

	/* Insert into chained list */

	pSocket->pChain = xmemo_InsertIntoChain ( &zSocketAnchor, pSocket ) ;
	if ( pSocket->pChain == NULL ) {
		xmemo_ErrorLog ( "xmemo_CreateUdpSocket", "xmemo_InsertIntoChain", NULL ) ;
		close ( pSocket->iSock ) ;
		XtFree ( (void*) pSocket ) ;
		return NULL ;
	}

	/* Add callback */

	pSocket->iCallbackReadId = XtAppAddInput ( xApp, pSocket->iSock,
	                                           (caddr_t) XtInputReadMask,
	                                           (XtInputCallbackProc) xmemo_RecvUdp,
	                                           (caddr_t) pSocket ) ;

	return pSocket ;

}

/******************************************************************************/
/*  xmemo_SelectUdpSocket                                                     */
/******************************************************************************/

static zSocketType* xmemo_SelectUdpSocket ( short iPortNumber )
{

zChainType	*pChain ;
zSocketType	*pSocket ;

	for ( pChain = zSocketAnchor.pHead ; pChain != NULL ; pChain = pChain->pNext ) {
		pSocket = (zSocketType*) pChain->pData ;
		if ( pSocket->iPort == iPortNumber ) return pSocket ;
	}

	return NULL ;

}

/******************************************************************************/
/*  xmemo_CreateListenTcpSocket                                               */
/******************************************************************************/

static zSocketType* xmemo_CreateListenTcpSocket ( zNetworkType *pNetwork )
{

char	sLog [ 64 ] ;
zSocketType	*pSocket ;
int 	rc ;
struct	sockaddr_in	addr ;
int 	bFlag ;

	if ( bMemoTraceLog ) {
		sprintf ( sLog, "%c://%s:%u",
			pNetwork->cProtocol,
			ipaddress_long_to_string ( pNetwork->iAddr[0] ),
			(unsigned short) pNetwork->iPort ) ;
		if ( pNetwork->cSearch == XMEMO_SEARCH_RANGE ) {
			strcat ( sLog, " - " ) ;
			strcat ( sLog, ipaddress_long_to_string ( pNetwork->iAddr[1] ) ) ;
		}
		xmemo_TraceLog ( "xmemo_CreateListenTcpSocket", sLog ) ;
	}

	/* Allocate memory */

	pSocket = (zSocketType*) XtMalloc ( sizeof(zSocketType) ) ;
	if ( pSocket == NULL ) {
		xmemo_ErrorLog ( "xmemo_CreateListenTcpSocket", "XtMalloc", NULL ) ;
		return NULL ;
	}

	pSocket->cSockType = XMEMO_SOCKET_TCP_LISTEN ;
	pSocket->cProtocol = pNetwork->cProtocol ;
	pSocket->iAddr = ntohl ( INADDR_ANY ) ;
	pSocket->iPort = pNetwork->iPort ;
	pSocket->tTimeout = 0 ;
	pSocket->iCallbackReadId   = 0 ;
	pSocket->iCallbackWriteId  = 0 ;
	pSocket->iCallbackExceptId = 0 ;

	/* Socket */

	pSocket->iSock = socket ( AF_INET, SOCK_STREAM, 0 ) ;
	if ( pSocket->iSock == 0 ) {
		xmemo_ErrorLog ( "xmemo_CreateListenTcpSocket", "socket", NULL ) ;
		XtFree ( (void*) pSocket ) ;
		return NULL ;
	}

	/* Bind */

	memset ( &addr, 0, sizeof(addr) ) ;
	addr.sin_family      = AF_INET ;
	addr.sin_port        = htons ( pSocket->iPort ) ;
	addr.sin_addr.s_addr = htonl ( pSocket->iAddr ) ;
	rc = bind ( pSocket->iSock, (struct sockaddr *)&addr, sizeof(addr) ) ;
	if ( rc != 0 ) {
		xmemo_ErrorLog ( "xmemo_CreateListenTcpSocket", "bind", NULL ) ;
		close ( pSocket->iSock ) ;
		XtFree ( (void*) pSocket ) ;
		return NULL ;
	}

	/* Listen */

	rc = listen ( pSocket->iSock, 5 ) ;
	if ( rc != 0 ) {
		xmemo_ErrorLog ( "xmemo_CreateListenTcpSocket", "listen", NULL ) ;
		close ( pSocket->iSock ) ;
		XtFree ( (void*) pSocket ) ;
		return NULL ;
	}

	/* Insert into chained list */

	pSocket->pChain = xmemo_InsertIntoChain ( &zSocketAnchor, pSocket ) ;
	if ( pSocket->pChain == NULL ) {
		xmemo_ErrorLog ( "xmemo_CreateUdpSocket", "xmemo_InsertIntoChain", NULL ) ;
		close ( pSocket->iSock ) ;
		XtFree ( (void*) pSocket ) ;
		return NULL ;
	}

	/* Add callback */

	pSocket->iCallbackReadId = XtAppAddInput ( xApp, pSocket->iSock,
	                                           (caddr_t) XtInputReadMask,
	                                           (XtInputCallbackProc) xmemo_AcceptTcp,
	                                           (caddr_t) pSocket ) ;

	return pSocket ;

}

/******************************************************************************/
/*  xmemo_CreateConnectTcpSocket                                              */
/******************************************************************************/

static zSocketType* xmemo_CreateConnectTcpSocket ( zMemberType* pMember, zMemoType* pMemo )
{

char	sLog [ 64 ] ;
zSocketType	*pSocket ;
int 	rc ;
struct	sockaddr_in	addr ;
int 	iFlag ;

	if ( bMemoTraceLog ) {
		sprintf ( sLog, "%c://%s:%u",
			pMember->cProtocol,
			ipaddress_long_to_string ( pMember->iAddr ),
			(unsigned short) pMember->iPort ) ;
		xmemo_TraceLog ( "xmemo_CreateConnectTcpSocket", sLog ) ;
	}

	/* Allocate memory */

	pSocket = (zSocketType*) XtMalloc ( sizeof(zSocketType) ) ;
	if ( pSocket == NULL ) {
		xmemo_ErrorLog ( "xmemo_CreateConnectTcpSocket", "XtMalloc", NULL ) ;
		return NULL ;
	}

	pSocket->cSockType = XMEMO_SOCKET_TCP_CONNECT ;
	pSocket->cProtocol = pMember->cProtocol ;
	pSocket->iAddr = pMember->iAddr ;
	pSocket->iPort = pMember->iPort ;
	pSocket->tTimeout = 0 ;
	pSocket->iCallbackReadId   = 0 ;
	pSocket->iCallbackWriteId  = 0 ;
	pSocket->iCallbackExceptId = 0 ;

	/* Socket */

	pSocket->iSock = socket ( AF_INET, SOCK_STREAM, 0 ) ;
	if ( pSocket->iSock == 0 ) {
		xmemo_ErrorLog ( "xmemo_CreateConnectTcpSocket", "socket", NULL ) ;
		XtFree ( (void*) pSocket ) ;
		return NULL ;
	}

	/* Non-blocking I/O */

	iFlag = 1 ;
	ioctl ( pSocket->iSock, FIONBIO, &iFlag ) ;

	/* Connect */

	memset ( &addr, 0, sizeof(addr) ) ;
	addr.sin_family      = AF_INET ;
	addr.sin_port        = htons ( pSocket->iPort ) ;
	addr.sin_addr.s_addr = htonl ( pSocket->iAddr ) ;
	rc = connect ( pSocket->iSock, (struct sockaddr *)&addr, sizeof(addr) ) ;
	if ( rc != 0  &&  errno != EINPROGRESS ) {
		xmemo_ErrorLog ( "xmemo_CreateConnectTcpSocket", "connect", NULL ) ;
		close ( pSocket->iSock ) ;
		XtFree ( (void*) pSocket ) ;
		return NULL ;
	}

	/* Insert into chained list */

	pSocket->pChain = xmemo_InsertIntoChain ( &zSocketAnchor, pSocket ) ;
	if ( pSocket->pChain == NULL ) {
		xmemo_ErrorLog ( "xmemo_CreateConnectTcpSocket", "xmemo_InsertIntoChain", NULL ) ;
		close ( pSocket->iSock ) ;
		XtFree ( (void*) pSocket ) ;
		return NULL ;
	}

	/* Add callback */

	pSocket->iCallbackReadId = XtAppAddInput ( xApp, pSocket->iSock,
	                                           (caddr_t) XtInputReadMask,
	                                           (XtInputCallbackProc) xmemo_RecvConnectedTcp,
	                                           (caddr_t) pMember ) ;

	pSocket->iCallbackWriteId = XtAppAddInput ( xApp, pSocket->iSock,
	                                            (caddr_t) XtInputWriteMask,
	                                            (XtInputCallbackProc) xmemo_ConnectTcp,
	                                            (caddr_t) pMember ) ;

	pMember->iState     = XMEMO_STATE_ANY_CONNECTING ;
	pMember->pTcpSocket = pSocket ;

	/* Start timer */

	time ( &(pMember->tTimeout) ) ;
	pMember->tTimeout += XMEMO_TIMEOUT_CONNECT ;
	xmemo_StartTimer ( pMember->tTimeout ) ;

	return pSocket ;

}

/******************************************************************************/
/*  xmemo_DeleteUdpSocket                                                     */
/******************************************************************************/

void xmemo_DeleteUdpSocket ( zSocketType * pSocket )
{

	if ( bMemoTraceLog ) {
		char	sLog [ 64 ] ;
		sprintf ( sLog, "%c://%s:%u    Type=%c",
			pSocket->cProtocol,
			ipaddress_long_to_string ( pSocket->iAddr ),
			pSocket->iPort, pSocket->cSockType ) ;
		xmemo_TraceLog ( "xmemo_DeleteUdpSocket", sLog ) ;
	}

	/* Remove callback */

	if ( pSocket->iCallbackReadId   != 0 ) XtRemoveInput ( pSocket->iCallbackReadId   ) ;
	if ( pSocket->iCallbackWriteId  != 0 ) XtRemoveInput ( pSocket->iCallbackWriteId  ) ;
	if ( pSocket->iCallbackExceptId != 0 ) XtRemoveInput ( pSocket->iCallbackExceptId ) ;

	/* Close socket */

	close ( pSocket->iSock ) ;

	/* Remove from chained list */

	if ( pSocket->pChain != NULL ) {
		xmemo_RemoveFromChain ( &zSocketAnchor, pSocket->pChain ) ;
	}

	/* Free allocated memory */

	XtFree ( (void*) pSocket ) ;

}

/******************************************************************************/
/*  xmemo_DeleteTcpSocket                                                     */
/******************************************************************************/

void xmemo_DeleteTcpSocket ( zSocketType * pSocket )
{

	if ( bMemoTraceLog ) {
		char	sLog [ 64 ] ;
		sprintf ( sLog, "%c://%s:%u    Type=%c",
			pSocket->cProtocol,
			ipaddress_long_to_string ( pSocket->iAddr ),
			pSocket->iPort, pSocket->cSockType ) ;
		xmemo_TraceLog ( "xmemo_DeleteTcpSocket", sLog ) ;
	}

	/* Remove callback */

	if ( pSocket->iCallbackReadId   != 0 ) XtRemoveInput ( pSocket->iCallbackReadId   ) ;
	if ( pSocket->iCallbackWriteId  != 0 ) XtRemoveInput ( pSocket->iCallbackWriteId  ) ;
	if ( pSocket->iCallbackExceptId != 0 ) XtRemoveInput ( pSocket->iCallbackExceptId ) ;

	/* Close socket */

	close ( pSocket->iSock ) ;

	/* Remove from chained list */

	if ( pSocket->pChain != NULL ) {
		xmemo_RemoveFromChain ( &zSocketAnchor, pSocket->pChain ) ;
	}

	/* Free allocated memory */

	XtFree ( (void*) pSocket ) ;

}

/******************************************************************************/
/*  xmemo_NetSend                                                             */
/******************************************************************************/

static int xmemo_NetSend ( zMemberType* pMember, zMemoType* pMemo )
{

	switch ( pMember->cProtocol ) {
	case XMEMO_PROTOCOL_HUSEN :
		pMember->pUdpSocket = xmemo_SelectUdpSocket ( pMember->iPort ) ;
		if ( pMember->pUdpSocket != NULL ) {
			xmemo_NetSend_Husen    ( pMember, pMemo ) ;
		}
		break ;
	case XMEMO_PROTOCOL_IPMSG :
		pMember->pUdpSocket = xmemo_SelectUdpSocket ( pMember->iPort ) ;
		if ( pMember->pUdpSocket != NULL ) {
			xmemo_NetSend_Ipmsg    ( pMember, pMemo ) ;
		}
		break ;
	case XMEMO_PROTOCOL_OBOEGAKI :
		xmemo_CreateConnectTcpSocket ( pMember, pMemo ) ;
		break ;
	}

}

/******************************************************************************/
/*  xmemo_NetSend_MemberList                                                  */
/******************************************************************************/

int xmemo_NetSend_MemberList ( zMemoType* pMemo )
{

zChainType	*pChain ;
zMemoType	*pCopyMemo ;
zMemberType	*pMember, *pSendPeer ;

	/* Copy memo */

	pCopyMemo = xmemo_CopyMemo ( pMemo ) ;
	if ( pCopyMemo == NULL ) {
		xmemo_ErrorLog ( "xmemo_NetSend_MemberList", "xmemo_CopyMemo", NULL ) ;
		return RC_ERROR ;
	}

	/* Insert into chained list */

	pCopyMemo->pChain = xmemo_InsertIntoChain ( &zSendMemoAnchor, pCopyMemo ) ;
	if ( pCopyMemo->pChain == NULL ) {
		xmemo_ErrorLog ( "xmemo_NetSend_MemberList", "xmemo_InsertIntoChain", NULL ) ;
		return RC_ERROR ;
	}

	/* Copy selected members */

	for ( pChain = zMemberAnchor.pHead ; pChain != NULL ; pChain = pChain->pNext ) {

		pMember = (zMemberType*) pChain->pData ;
		if ( ! pMember->bSelect ) continue ;

		/* Allocate memory */

		pSendPeer = (zMemberType*) XtCalloc ( sizeof(zMemberType), 1 ) ;
		if ( pSendPeer == NULL ) {
			xmemo_ErrorLog ( "xmemo_NetSend_MemberList", "XtCalloc", NULL ) ;
			if ( pCopyMemo->iSendCount == 0 ) {
				xmemo_RemoveFromChain ( &zSendMemoAnchor, pCopyMemo->pChain ) ;
				xmemo_FreeMemo ( pCopyMemo ) ;
			}
			return RC_ERROR ;
		}
		pSendPeer->cProtocol = pMember->cProtocol ;
		pSendPeer->iAddr     = pMember->iAddr ;
		pSendPeer->iPort     = pMember->iPort ;
		pSendPeer->cCypher   = pMember->cCypher ;
		pSendPeer->iState    = XMEMO_STATE_ANY_IDLE ;
		pSendPeer->iRetry    = 0 ;
		pSendPeer->pSendMemo = pCopyMemo ;

		/* Insert into chained list */

		pSendPeer->pChain = xmemo_InsertIntoChain ( &zSendPeerAnchor, pSendPeer ) ;
		if ( pSendPeer->pChain == NULL ) {
			xmemo_ErrorLog ( "xmemo_NetSend_MemberList", "xmemo_InsertIntoChain", NULL ) ;
			xmemo_FreeMember ( pSendPeer ) ;
			if ( pCopyMemo->iSendCount == 0 ) {
				xmemo_RemoveFromChain ( &zSendMemoAnchor, pCopyMemo->pChain ) ;
				xmemo_FreeMemo ( pCopyMemo ) ;
			}
			return RC_ERROR ;
		}

		pCopyMemo->iSendCount ++ ;

	}

	/* Send memo to peers */

	pChain = zSendPeerAnchor.pHead ;
	while ( pChain != NULL ) {

		pSendPeer = (zMemberType*) pChain->pData ;
		pChain = pChain->pNext ;

		if ( pSendPeer->iState == XMEMO_STATE_ANY_IDLE ) {
			xmemo_NetSend ( pSendPeer, pCopyMemo ) ;
		}

	}

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_NetNotify                                                           */
/******************************************************************************/

int xmemo_NetNotify ( zNetworkType* pNetwork, int iNotify )
{

char	sLog [ 64 ] ;
int 	rc ;
zChainType	*pChain ;
zSocketType	*pSocket ;
int 	iUdpSock ;
long	iUdpAddr ;
short	iUdpPort ;

	if ( bMemoTraceLog ) {
		sprintf ( sLog, "%c://%s:%u",
			pNetwork->cProtocol,
			ipaddress_long_to_string ( pNetwork->iAddr[0] ),
			(unsigned short) pNetwork->iPort ) ;
		if ( pNetwork->cSearch == XMEMO_SEARCH_RANGE ) {
			strcat ( sLog, " - " ) ;
			strcat ( sLog, ipaddress_long_to_string ( pNetwork->iAddr[1] ) ) ;
		}
		xmemo_TraceLog ( "xmemo_NetNotify", sLog ) ;
	}

	if ( pNetwork->pUdpSock == NULL ) {

		/* Search socket */
		for ( pChain = zSocketAnchor.pHead ; pChain != NULL ; pChain = pChain->pNext ) {
			pSocket = (zSocketType*) ( pChain->pData ) ;
			if ( pNetwork->iPort == pSocket->iPort ) break ;
		}

		/* If not found, Create socket */
		if ( pChain == NULL ) {
			pNetwork->pUdpSock = xmemo_CreateUdpSocket ( pNetwork ) ;
			if ( pNetwork->pUdpSock == NULL ) {
				return RC_ERROR ;
			}
			if ( pNetwork->cProtocol == XMEMO_PROTOCOL_OBOEGAKI ) {
				pNetwork->pTcpSock = xmemo_CreateListenTcpSocket ( pNetwork ) ;
				if ( pNetwork->pTcpSock == NULL ) {
					return RC_ERROR ;
				}
			}
		} else {
			pNetwork->pUdpSock = pSocket ;
		}

		pNetwork->bActive = TRUE ;

	}

	for ( iUdpAddr = pNetwork->iAddr[0] ; ; iUdpAddr ++ ) {
		iUdpSock = pNetwork->pUdpSock->iSock ;
		iUdpPort = pNetwork->iPort ;
		switch ( pNetwork->cProtocol ) {
		case XMEMO_PROTOCOL_HUSEN :
			rc = xmemo_NetNotify_Husen    ( iUdpSock, iUdpAddr, iUdpPort, iNotify ) ; break ;
		case XMEMO_PROTOCOL_IPMSG :
			rc = xmemo_NetNotify_Ipmsg    ( iUdpSock, iUdpAddr, iUdpPort, iNotify ) ; break ;
		case XMEMO_PROTOCOL_OBOEGAKI :
			rc = xmemo_NetNotify_Oboegaki ( iUdpSock, iUdpAddr, iUdpPort, iNotify ) ; break ;
		}
		if ( pNetwork->cSearch != XMEMO_SEARCH_RANGE || iUdpAddr == pNetwork->iAddr[1] ) break ;
	}

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_NetNotify_Broadcast                                                 */
/******************************************************************************/

int xmemo_NetNotify_Broadcast ( int iNotify )
{

zChainType		*pChain ;
zNetworkType*	pNetwork ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_NetNotify_Broadcast", NULL ) ;

	for ( pChain = zNetworkAnchor.pHead ; pChain != NULL ; pChain = pChain->pNext ) {

		pNetwork = (zNetworkType*) ( pChain->pData ) ;

		if ( pNetwork->bEnable ) xmemo_NetNotify ( pNetwork, iNotify ) ;

	}

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_InitNetwork                                                         */
/******************************************************************************/

int xmemo_InitNetwork ( )
{

struct	hostent	*pHost ;
int 	rc ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_InitNetwork", NULL ) ;

	/* Local host */

	rc = gethostname ( sMemoHostname, MAXLEN_HOSTNAME ) ;
	if ( rc == 0 ) {
		pHost = (struct hostent *) gethostbyname ( sMemoHostname ) ;
		if ( pHost != NULL ) {
			iMemoHostaddr = ntohl ( *(long*)pHost->h_addr_list[0] ) ;
			strcpy ( sMemoHostaddr, ipaddress_long_to_string ( iMemoHostaddr ) ) ;
		}
	} else {
		strncpy ( sMemoHostname, "unknown", MAXLEN_HOSTNAME ) ;
	}

	/* Notify to the network */

	xmemo_NetNotify_Broadcast ( XMEMO_NET_NOTIFY_ENTER ) ;

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  xmemo_ExitNetwork                                                         */
/******************************************************************************/

int xmemo_ExitNetwork ( )
{

zChainType	*pChain ;
zSocketType	*pSocket ;

	if ( bMemoTraceLog ) xmemo_TraceLog ( "xmemo_ExitNetwork", NULL ) ;

	/* Notify to the network */

	xmemo_NetNotify_Broadcast ( XMEMO_NET_NOTIFY_LEAVE ) ;

	/* Delete all sockets */

	pChain = zSocketAnchor.pHead ;
	while ( pChain != NULL ) {

		pSocket = (zSocketType*) ( pChain->pData ) ;
		pChain = pChain->pNext ;

		if ( pSocket->cSockType == XMEMO_SOCKET_UDP ) {
			xmemo_DeleteUdpSocket ( pSocket ) ;
		} else {
			xmemo_DeleteTcpSocket ( pSocket ) ;
		}

	}

	/* Free network list */

	xmemo_FreeNetworkAll ( ) ;

	return RC_SUCCESS ;

}

/******************************************************************************/
/*  ipaddress_string_to_long                                                  */
/******************************************************************************/

long ipaddress_string_to_long ( char *pAddress )
{

	return ntohl ( inet_addr ( pAddress ) ) ;

}

/******************************************************************************/
/*  ipaddress_long_to_string                                                  */
/******************************************************************************/

char* ipaddress_long_to_string ( long iAddress )
{

static	char	sAddress [ 16 ] ;
struct	in_addr	ipaddress ;

	*((int*)&ipaddress) = htonl( iAddress ) ;
	strcpy ( sAddress, inet_ntoa(ipaddress) ) ;

	return sAddress ;

}

/******************************************************************************/
/*  int_to_lobyte_first                                                       */
/******************************************************************************/

int int_to_lobyte_first ( int i )
{

union {
	int 	iInt ;
	char	cByte[4] ;
} zByteOrder ;
char	cSwap ;

	zByteOrder.iInt = 0x11223344 ;
	if ( zByteOrder.cByte[0] == 0x44 ) {
		return i ;
	} else {
		zByteOrder.iInt = i ;
		cSwap               = zByteOrder.cByte[0] ;
		zByteOrder.cByte[0] = zByteOrder.cByte[3] ;
		zByteOrder.cByte[3] = cSwap ;
		cSwap               = zByteOrder.cByte[1] ;
		zByteOrder.cByte[1] = zByteOrder.cByte[2] ;
		zByteOrder.cByte[2] = cSwap ;
		return zByteOrder.iInt ;
	}

}

/******************************************************************************/
/*  int_to_hibyte_first                                                       */
/******************************************************************************/

int int_to_hibyte_first ( int i )
{

union {
	int 	iInt ;
	char	cByte[4] ;
} zByteOrder ;
char	cSwap ;

	zByteOrder.iInt = 0x11223344 ;
	if ( zByteOrder.cByte[0] == 0x11 ) {
		return i ;
	} else {
		zByteOrder.iInt = i ;
		cSwap               = zByteOrder.cByte[0] ;
		zByteOrder.cByte[0] = zByteOrder.cByte[3] ;
		zByteOrder.cByte[3] = cSwap ;
		cSwap               = zByteOrder.cByte[1] ;
		zByteOrder.cByte[1] = zByteOrder.cByte[2] ;
		zByteOrder.cByte[2] = cSwap ;
		return zByteOrder.iInt ;
	}

}

/******************************************************************************/
/*  int_from_lobyte_first                                                     */
/******************************************************************************/

int int_from_lobyte_first ( int i )
{
	return int_to_lobyte_first ( i ) ;
}

/******************************************************************************/
/*  int_from_hibyte_first                                                     */
/******************************************************************************/

int int_from_hibyte_first ( int i )
{
	return int_to_hibyte_first ( i ) ;
}
